from BlogPost.Entity import BlogPost
from User.Business.UserManager import UserManager

from lawrence.base import EntityManager

class BlogPostManager (EntityManager):
	__entity__ = BlogPost

	def __init__(self, **kwargs):
		EntityManager.__init__(self, **kwargs)
		self.UserManager = UserManager(**kwargs)

	def GetById(self, blogPostId):
		blogPostRow = self.DatabaseCursor.Single('''
			SELECT
				BlogPost.Id,
				BlogPost.Content,
				BlogPost.CreationDateTime,
				BlogPost.AuthorId AS `Author.Id`
			FROM
				BlogPost
			WHERE
				BlogPost.Id = %s''',
			[blogPostId])
		blogPost = BlogPost(**blogPostRow)
		blogPost.Author = self.UserManager.GetById(blogPost.Author.Id)

		return blogPost

	def GetLatest(self, number = 10):
		return [self.GetById(row.Id) for row in self.DatabaseCursor.Execute('''
			SELECT Id
			FROM BlogPost
			ORDER BY CreationDateTime DESC
			LIMIT %s''',
			(number, ))]

	def Delete(self, news):
		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute('''
			DELETE FROM BlogPost
			WHERE Id = %s''',
			(news.Id, ))
		self.DatabaseCursor.Commit()

	def Create(self, news):
		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute('''
			INSERT INTO BlogPost (Content, AuthorId)
			VALUES (%s, %s)''',
			(news.Content,
			 news.Author.Id))
		news.Id = self.DatabaseCursor.ScopeIdentity()
		self.DatabaseCursor.Commit()
		return news
