import datetime
from Campus.Business import CampusManager
from Campus.Entity import Campus
from lawrence.base import Handler
from lawrence.HTTPStatus import *
import lawrence.util as util
import lawrence.config as config
import lawrence.db as db

DBRow = db.DBRow

class Index (Handler):
	__method__ = 'GET'
	__path__ = '/campus/'

	def Handle(self, *,
			ajax: bool = False,
			json: str = None):
		self.RequirePermission('/campus/', 'read')

		campusList = db.Execute('''
			SELECT Campus.Id AS id,
				   Campus.Name AS name
			FROM Campus
			WHERE ''' + db.query_in('Campus.Id', self.GetManager(CampusManager).GetAuthorized()) + '''
			ORDER BY Campus.Name''')
		if ajax or json == 'select':
			import json
			return json.dumps([{'value': campus.id, 'text': campus.name} for campus in campusList])
		for campus in campusList:
			campus.path = '/campus/%s/' % campus.id

		return self.RenderTemplate('/Campus/View/Index',
							 Campus = campusList)

class CampusNew (Handler):
	__method__ = 'POST'
	__path__ = '/campus/create/'

	def Handle(self, *,	Name: str, Address: str):
		self.RequirePermission('/campus/create/', 'write')

		self.DatabaseCursor.Begin()
		db.Execute(
			'''INSERT INTO Campus (Name, Address)
			   VALUES (%s, %s)''',
			(Name, Address))
		self.DatabaseCursor.Commit()

		raise SeeOther('/campus/')


class DetailView (Handler):
	__method__ = 'GET'
	__path__ = '/campus/(?P<campusId>[1-9][0-9]*)/'

	def Handle(self, *,
			campus: Campus):
		self.RequirePermission(campus.Path, 'read')

		examDateTimeList = self.DatabaseCursor.Execute('''
			SELECT ExamDateTime, Priority
			FROM CampusExamDateTime
			WHERE CampusId = %s
			ORDER BY Priority, ExamDateTime''',
			[campus.Id])

		return self.RenderTemplate('/Campus/View/Detail',
			campus = campus,
			examDateTimeList = examDateTimeList)

class CreateExamDateTime (Handler):
	__method__ = 'POST'
	__path__ = '/campus/(?P<campusId>[1-9][0-9]*)/ExamDateTime/'

	def Handle(self, *,
			campus: Campus,
			priority: int,
			examDate: datetime.date,
			examTime: datetime.time):
		self.RequirePermission(campus.Path, 'admin')

		examDateTime = datetime.datetime.combine(examDate, examTime)

		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute('''
			INSERT INTO CampusExamDateTime (CampusId, ExamDateTime, Priority)
			VALUES (%s, %s, %s)''',
			[campus.Id, examDateTime, priority])
		self.DatabaseCursor.Commit()

		raise SeeOther(campus.Path)

class DeleteExamDateTime (Handler):
	__method__ = 'POST'
	__path__ = '/campus/(?P<campusId>[1-9][0-9]*)/ExamDateTime/Delete/'

	def Handle(self, *,
			campus: Campus,
			examDateTime: datetime.datetime):
		self.RequirePermission(campus.Path, 'admin')

		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute('''
			DELETE FROM CampusExamDateTime
			WHERE CampusId = %s AND ExamDateTime = %s''',
			[campus.Id, examDateTime])
		self.DatabaseCursor.Commit()

		raise NoContent()

class DetailEdit (Handler):
	__method__ = 'POST'
	__path__ = '/campus/(?P<campusId>[1-9][0-9]*)/'

	def Handle(self, *,
			campus: Campus,
			name: str,
			value: str):
		self.RequirePermission(campus.Path, 'admin')

		if name not in ('name', 'address', 'exam_campus'):
			raise BadRequest()
		if name == 'exam_campus':
			examCampusId = int(value)
			self.DatabaseCursor.Begin()
			self.DatabaseCursor.Execute('''
				DELETE FROM ExamCampusMap
				WHERE CampusId = %s''',
				(campus.Id, ))
			if examCampusId != campus.Id:
				self.DatabaseCursor.Execute('''
					INSERT INTO ExamCampusMap (CampusId, ExamCampusId)
					VALUES (%s, %s)''',
					(campus.Id,
					 examCampusId))
			self.DatabaseCursor.Commit()
			raise NoContent()

		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute(
			'''UPDATE Campus
			   SET %s = %%s
			   WHERE id = %%s''' % (name, ),
			(value,
			 campus.Id))
		self.DatabaseCursor.Commit()

		raise NoContent()

class Map (Handler):
	__method__ = 'GET'
	__path__ = '/campus/(?P<campusId>[1-9][0-9]*)/map/'

	def Handle(self, *,
			campus: Campus,
			size: int = 300):
		mapsParams = {
			'markers': 'icon:http://%s/static/logo-mini.png|%s' % (config.domain, campus.address),
			'size': '%sx%s' % (size, size),
			'zoom': 15,
			'sensor': 'false',
		}
		raise Found('http://maps.googleapis.com/maps/api/staticmap' +
					util.QueryString(mapsParams))

class GoogleMaps (Handler):
	__method__ = 'GET'
	__path__ = '/campus/(?P<campusId>[1-9][0-9]*)/google-maps/'

	def Handle(self, *,
			campus: Campus):
		mapsParams = {
			'q': campus.address.encode('utf-8'),
		}
		raise Found('http://maps.google.com/' + util.QueryString(mapsParams))

