import datetime
from Candidate.Business.CandidateNegotiationManager import CandidateNegotiationManager
from Candidate.Controller.Candidate import DBRow
from Candidate.Entity.Candidate import Candidate
from Candidate.Entity.CandidateNegotiation import CandidateNegotiation
from User.Business.UserPermissionManager import UserPermissionManager
from User.Entity.UserPermission import UserPermission
from lawrence import parse as parse
from lawrence.HTTPStatus import NotFound, Forbidden, NoContent, BadRequest
from lawrence.base import Handler
from lawrence.HTTPStatus import *
import lawrence.util as util
import lawrence.config as config
import lawrence.db as db
import json

# class AppointmentAll (Handler):
# 	__method__ = 'GET'
# 	__path__ = '/[Aa]ppointments/'

# 	def Handle(self, *,
# 			Campus: List[int] = None,
# 			Date: List[datetime.date] = None,
# 			):

# 		if not self.GetManager(UserPermissionManager).HasPermission(
# 				self.CurrentUser,
# 				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment')):
# 			raise Forbidden()

# 		Date = Date or None
# 		Campus = Campus or None
# 		if Date is not None and Campus is not None:
# 			appointmentFilter = '''LEFT JOIN Candidate ON Candidate.Id = CandidateAppointment.CandidateId
# 									WHERE Candidate.CampusId IN %s
# 									AND AppointmentDateTime IN %s''', (str(tuple(Campus)), str(tuple(Date)))
# 		elif Date is not None:
# 			appointmentFilter = '''WHERE AppointmentDateTime IN %s''', (str(tuple(Date)))
# 		elif Campus is not None:
# 			appointmentFilter = '''LEFT JOIN Candidate ON Candidate.Id = CandidateAppointment.CandidateId
# 									WHERE Candidate.CampusId IN %s''', (str(tuple(Campus)))
# 		else:
# 			appointmentFilter = ""
# 		appointmentList = self.DatabaseCursor.Execute('''
# 			SELECT *
# 			FROM CandidateAppointment
# 			%s''',
# 			(appointmentFilter)

# 		return self.RenderTemplate('Candidate/View/AppointmentIndex', candidateList)


class AppointmentCheck (Handler):
	__method__ = 'GET'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/appointment/check/'

	def Handle(self, *,
			candidate: Candidate,
			date: datetime.date,
			time: datetime.time):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'CreateAppointment')):
			raise Forbidden()
		appointmentDateTime = datetime.datetime.combine(date, time)
		rows = self.DatabaseCursor.Execute('''
			SELECT count(1) as count
			FROM CandidateAppointment
			WHERE AppointmentDateTime = %s
			AND CandidateId <> %s''',
			(appointmentDateTime, candidate.Id))
		return json.dumps({'count': rows[0].count})


class AppointmentNew (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/appointment/=/new/'

	def Handle(self, *,
			candidate: Candidate,
			date: datetime.date,
			time: datetime.time):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'CreateAppointment')):
			raise Forbidden()

		self.RequirePermission(candidate.Campus.Path, 'read')

		appointmentDateTime = datetime.datetime.combine(date, time)
		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute('''
			DELETE FROM CandidateAppointment
			WHERE
				CandidateId = %s AND
				UserId IS NULL AND
				DATE(AppointmentDateTime) >= DATE(NOW())''',
			(candidate.Id, ))
		try:
			self.DatabaseCursor.Single('''
				SELECT 1
				FROM CandidateAppointment
				WHERE CandidateId = %s AND AppointmentDateTime = %s''',
				(candidate.Id, appointmentDateTime))
			# FIXME: throw some error here
		except ValueError:
			self.DatabaseCursor.Execute(
				'''INSERT INTO CandidateAppointment (CandidateId, AppointmentDateTime)
				   VALUES (%s, %s)''',
				(candidate.Id,
				 appointmentDateTime))
		self.DatabaseCursor.Commit()

		activity = DBRow(type = 'appointment',
						 user = DBRow(id = None, name = None),
						 datetime = appointmentDateTime,
						 student_name = candidate.student_name)
		self.Model.CanDeleteAppointment = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'DeleteAppointment'))
		self.Model.CanServiceAppointment = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment'))
		return self.RenderTemplate('Candidate/View/Candidate/_Fields:candidate_activity', candidate, activity)


class AppointmentDelete (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/appointment/=/delete/'

	def Handle(self, *,
			candidate: Candidate,
			appointmentDateTime: datetime.datetime):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'DeleteAppointment')):
			raise Forbidden()

		self.RequirePermission(candidate.Campus.Path, 'read')

		self.DatabaseCursor.begin()
		try:
			appointment = self.DatabaseCursor.Single('''
				SELECT UserId FROM CandidateAppointment
				WHERE CandidateId = %s AND
					  AppointmentDateTime = %s''',
				(candidate.Id,
				 appointmentDateTime))
		except ValueError:
			raise NotFound()
		if appointment.UserId is not None or appointmentDateTime.date() < datetime.date.today():
			self.RequirePermission(candidate.path, 'admin')

		self.DatabaseCursor.Execute('''
			DELETE FROM CandidateAppointment
			WHERE CandidateId = %s AND
				  AppointmentDateTime = %s''',
			(candidate.Id,
			 appointmentDateTime))
		self.DatabaseCursor.Commit()

		raise NoContent()


class AppointmentView (Handler):
	__method__ = 'GET'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/appointment/'

	def Handle(self, *,
			candidate: Candidate):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment')):
			raise Forbidden()

		try:
			appointment = self.DatabaseCursor.Single('''
				SELECT AppointmentDateTime
				FROM CandidateAppointment
				WHERE CandidateId = %s AND
					  DATE(AppointmentDateTime) = DATE(NOW()) AND
					  UserId IS NULL
				ORDER BY AppointmentDateTime
				LIMIT 1''',
				(candidate.Id, ))
		except ValueError:
			raise NotFound()

		schools = self.DatabaseCursor.Execute('''
			SELECT SchoolId AS Id, Name
			FROM School
			ORDER BY Name''')

		self.Model.CanEdit = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'Edit'))
		self.Model.CanPostComment = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'PostComment'))
		self.Model.CanPostRating = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'PostRating'))
		self.Model.CanDeleteNegotiation = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'DeleteNegotiation'))
		self.Model.CanImportGrades = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ImportGrades'))
		self.Model.CanServiceAppointment = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment'))
		self.Model.Candidate = candidate

		return self.RenderTemplate('Candidate/View/Candidate/Appointment',
							 candidate = candidate,
							 appointment = appointment,
							 schools = schools)


class Appointment (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/appointment/'

	def Handle(self, *,
			candidate: Candidate,
			name: str,
			value: str):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment')):
			raise Forbidden()
		self.RequirePermission(candidate.Campus.Path, 'read')

		self.DatabaseCursor.begin()
		if name == 'class':
			try:
				value = int(value)
			except ValueError:
				raise BadRequest()
			self.DatabaseCursor.Execute('''
				INSERT INTO CandidateNegotiation (CandidateId, SchoolClassId, LowerBound, UpperBound,
							NegotiationDate, ExpirationDate, FullNumber)
				VALUES (%s, %s, NULL, NULL, CURRENT_DATE, NULL, NULL)''',
				(candidate.Id, value))
		elif name.startswith('LowerBound-') or name.startswith('UpperBound-') or name.startswith('FullNumber-'):

			name, schoolClassId = name.split("-")
			schoolClassId = int(schoolClassId)
			if name not in ('LowerBound', 'UpperBound', 'FullNumber'):
				raise BadRequest()
			try:
				if name != 'FullNumber':
					value = parse.percentage(value)
				else:
					value = parse.int_(value)
			except ValueError:
				value = None
			try:
				self.DatabaseCursor.Single('''
					SELECT 1
					FROM CandidateNegotiation
					WHERE CandidateId = %s AND
						  SchoolClassId = %s AND
						  NegotiationDate = CURRENT_DATE''',
					(candidate.Id,
					 schoolClassId))
			except ValueError:
				self.DatabaseCursor.Execute('''
					INSERT INTO CandidateNegotiation (CandidateId, SchoolClassId, LowerBound, UpperBound,
							NegotiationDate, ExpirationDate, FullNumber)
					VALUES (%s, %s, NULL, NULL, CURRENT_DATE, NULL, NULL)''',
					(candidate.Id,
					 schoolClassId))
			self.DatabaseCursor.Execute('''
				UPDATE CandidateNegotiation
				SET %s = %%s
				WHERE CandidateId = %%s AND
					  SchoolClassId = %%s AND
					  NegotiationDate = CURRENT_DATE''' % name,
				(value,
				 candidate.Id,
				 schoolClassId))
		elif name == 'user':
			# FIXME: kill this code path; this causes trouble when there are two appointments on the same day.
			try:
				appointment = self.DatabaseCursor.Single('''
					SELECT AppointmentDateTime
					FROM CandidateAppointment
					WHERE CandidateId = %s AND
						  DATE(AppointmentDateTime) = DATE(NOW())''',
					(candidate.Id, ))
			except ValueError:
				raise NotFound()
			appointmentDateTime = appointment.AppointmentDateTime
			self.DatabaseCursor.Execute('''
				UPDATE CandidateAppointment
				SET UserId = %s
				WHERE CandidateId = %s AND
					  AppointmentDateTime = %s''',
				(value,
				 candidate.Id,
				 appointmentDateTime))
		else:
			raise BadRequest()
		self.DatabaseCursor.Commit()
		raise NoContent()


class NegotiationDelete (Handler):
	__method__ = 'POST'
	__path__ = '/Candidate/Negotiation/(?P<negotiationId>[1-9][0-9]*)/Delete/'

	def Handle(self, *,
			candidateNegotiation: CandidateNegotiation):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'DeleteNegotiation')):
			raise Forbidden()

		self.RequirePermission(candidateNegotiation.candidate.Campus.Path, 'read')

		self.GetManager(CandidateNegotiationManager).Delete(candidateNegotiation)

		raise NoContent()


class ConfirmAppointment (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/appointment/(?P<appointmentId>[1-9][0-9]*)/'

	def Handle(self, *,
			candidate: Candidate,
			appointmentId: int,
			confirm: bool = False):
		self.RequirePermission(candidate.Campus.Path, 'read')

		if confirm:
			Candidate.Appointment.get(appointmentId).confirm(self.CurrentUser)

		raise NoContent()


class ValidTimeView (Handler):
	__method__ = 'GET'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/appointment/=/valid-time/'

	def Handle(self, *,
			candidate: Candidate,
			date: datetime.date):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'View')):
			raise Forbidden()

		self.RequirePermission(candidate.Campus.Path, 'read')

		return self.RenderTemplate('Candidate/View/Candidate/_ValidAppointmentTimes',
								   candidate = candidate,
								   date = date)
