import datetime
from Candidate.Business.CandidateManager import CandidateManager
from Candidate.Entity.Candidate import Candidate
from Candidate.Entity.CandidateFilter import CandidateFilter
from User.Business.UserPermissionManager import UserPermissionManager
from User.Entity.UserPermission import UserPermission
from lawrence import config as config
from lawrence.HTTPStatus import Forbidden, NoContent
from lawrence.base import Handler
import lawrence.mail

class BulkEmail (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/=/email/'

	def Handle(self, *,
			AppointmentStatus: str,
			Course: int,
			Campus: int,
			ExamDate: datetime.date,
			ExamTime: datetime.time,
			Tag: int,
			Query: str,
			SortOrder: str,
			Status: str,
			eMailSubject: str,
			eMailBody: str):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'EMail')):
			raise Forbidden()

		candidateFilter = CandidateFilter(
			Page = 1,
			Campus = Campus,
			Course = Course,
			Status = Status,
			AppointmentStatus = AppointmentStatus,
			ExamDate = ExamDate,
			ExamTime = ExamTime,
			Tag = Tag)

		candidateList = Candidate.Search(user = self.CurrentUser, candidateFilter = candidateFilter)
		for candidate in candidateList:
			candidate = self.GetManager(CandidateManager).GetById(candidate.Id)
			emailBody = "Subject: " + eMailSubject + "\n\n" + eMailBody
			emailBody = emailBody.replace("{nome do aluno}", candidate.student_name)
			emailBody = emailBody.replace("{nome da unidade}", candidate.Campus.Name)
			emailBody = emailBody.replace("{nome da turma}", candidate.Course.Name)
			lawrence.mail.Queue(config.candidate_email, candidate.contact_email, emailBody)
		raise NoContent()
