from collections import namedtuple
from io import BytesIO
from openpyxl import load_workbook
from Candidate.Entity.Candidate import Candidate
from User.Business.UserPermissionManager import UserPermissionManager
from User.Entity.UserPermission import UserPermission
from lawrence import util as util, db
from lawrence.HTTPStatus import Forbidden, SeeOther
from lawrence.base import Handler

class ImportBulkGrades (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/=/grades/'

	def ImportRow(self, worksheet, row):
		ExcelImportError = namedtuple('ExcelImportError', ['cell', 'value', 'error_type'])
		ref = 'A%d' % row
		try:
			candidateId = int(worksheet.cell(ref).value)
			if not 10**7 <= candidateId < 10**8:
				raise ValueError
			candidateId = util.DecodeId(candidateId)
			campus = self.DatabaseCursor.Single('''
				SELECT ExamCampusId AS ExamCampusId
				FROM Candidate
				WHERE Id = %s AND status != 'cancelled' ''',
				(candidateId, ))
			self.SeenExamCampusSet.add(campus.ExamCampusId)
		except (ValueError, TypeError):
			return [ExcelImportError(cell = ref,
								 value = worksheet.cell(ref).value,
								 error_type = 'invalid_id')]

		grades = {}
		errors = []
		for col in self.Subjects:
			try:
				ref = '%s%d' % (col, row)
				grades[self.Subjects[col]] = int(worksheet.cell(ref).value)
			except (ValueError, TypeError):
				errors.append(ExcelImportError(cell = ref,
										   value = worksheet.cell(ref).value,
										   error_type = 'not_integer'))
				grades[self.Subjects[col]] = 0
		self.DatabaseCursor.Execute('''
			DELETE FROM CandidateGrade WHERE CandidateId = %s''',
			(candidateId, ))
		self.DatabaseCursor.Execute('''
			UPDATE Candidate
			SET status = 'present'
			WHERE id = %s AND
				  status IN ('unconfirmed', 'confirmed', 'absent', 'failed')''',
			(candidateId, ))
		for subjectName, grade in list(grades.items()):
			self.DatabaseCursor.Execute(
				'''INSERT INTO CandidateGrade
					 (CandidateId, subject_name, grade)
				   VALUES (%s, %s, %s)''',
				(candidateId, subjectName, grade))
		self.ImportedRows += 1
		return errors

	def Handle(self, *,
			grades: bytes):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ImportGrades')):
			raise Forbidden()

		spreadsheetFile = BytesIO(grades)
		workbook = load_workbook(spreadsheetFile)
		self.Model.WorksheetNotFound = False
		worksheet = workbook.get_sheet_by_name(name = "Notas")
		if worksheet is None:
			self.Model.WorksheetNotFound = True
			return self.RenderTemplate('Candidate/View/Candidate/Grades/ImportBulk')
		self.SeenExamCampusSet = set()
		self.ImportErrors = []
		self.Subjects = {}
		for i in range(1, 26):  # FIXME
			col = chr(ord('A') + i)  # FIXME
			ref = '%s1' % (col, )
			if not worksheet.cell(ref).value:
				break
			self.Subjects[col] = worksheet.cell(ref).value
		self.ImportedRows = 0
		rowId = 2
		self.DatabaseCursor.begin()
		while True:
			if not worksheet.cell('A%d' % rowId).value:
				break
			self.ImportErrors.extend(self.ImportRow(worksheet, rowId))
			rowId += 1
		self.DatabaseCursor.Execute('''
			UPDATE Candidate
			SET status = 'absent'
			WHERE DATE(ExamDateTime) <= DATE(TIMESTAMPADD(HOUR, -6, NOW())) AND
				  status IN ('unconfirmed', 'confirmed', 'absent') AND ''' + \
				  db.query_in('ExamCampusId', tuple(self.SeenExamCampusSet)))
		self.DatabaseCursor.Commit()
		return self.RenderTemplate('Candidate/View/Candidate/Grades/ImportBulk',
							 import_errors = self.ImportErrors,
							 imported_rows = self.ImportedRows)


class ImportCandidateGrades (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/grades/'

	def Handle(self, *,
			candidate: Candidate,
			grades_Matemática: int,
			grades_Português: int):

		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ImportGrades')):
			raise Forbidden()

		self.DatabaseCursor.begin()
		self.DatabaseCursor.Execute('''
			DELETE FROM CandidateGrade WHERE CandidateId = %s''',
			(candidate.Id, ))
		self.DatabaseCursor.Execute('''
			UPDATE Candidate
			SET ExamDateTime = NULL
			WHERE id = %s''',
			(candidate.Id, ))
		if candidate.status in ('unconfirmed', 'confirmed', 'absent'):
			self.DatabaseCursor.Execute('''
				UPDATE Candidate
				SET status = 'present'
				WHERE id = %s''',
				(candidate.Id, ))
			self.DatabaseCursor.Execute('''
				INSERT INTO audit_candidate_status
					(CandidateId, OldValue, NewValue, UserId)
				VALUES (%s, %s, 'present', %s)''',
				(candidate.Id,
				 candidate.status,
				 self.CurrentUser.id))
		self.DatabaseCursor.Execute(
			'''INSERT INTO CandidateGrade
				 (CandidateId, subject_name, grade)
			   VALUES (%s, %s, %s)''',
			(candidate.Id,
			 "Matemática",
			 grades_Matemática))
		self.DatabaseCursor.Execute(
			'''INSERT INTO CandidateGrade
				 (CandidateId, subject_name, grade)
			   VALUES (%s, %s, %s)''',
			(candidate.Id,
			 "Português",
			 grades_Português))
		self.DatabaseCursor.Commit()
		raise SeeOther(candidate.path)