from io import BytesIO
import datetime
from Candidate.Entity.Candidate import Candidate
from User.Business.UserPermissionManager import UserPermissionManager
from User.Entity.UserPermission import UserPermission
from lawrence import parse as parse
from lawrence.HTTPStatus import NotFound, Forbidden, NoContent, BadRequest, SeeOther
from lawrence.base import Handler
from lawrence.HTTPStatus import *
import lawrence.util as util
import lawrence.config as config
import lawrence.db as db
import json
import csv
import os
from openpyxl import Workbook
from openpyxl.reader.excel import load_workbook
from Crypto.Hash import SHA256

class QSERegistrationView (Handler):
	__method__ = 'GET'
	__path__ = '/candidate/=/registrationqse/'

	def Handle(self, *,
			course: int = None):

		if course is None:
			courseList = self.DatabaseCursor.Execute('''
					SELECT
						Course.id AS Id,
						Course.Name AS Name,
						CourseSegment.Name AS SegmentName
					FROM
						Course
						INNER JOIN CourseSegment
						  ON Course.SegmentId = CourseSegment.id
					WHERE
						Course.id IN (SELECT CourseId FROM SchoolClass)
					ORDER BY CourseSegment.SortOrder, Course.SortOrder, Course.Name''')
			return self.RenderTemplate('Candidate/View/Candidate/Registration/Step-3',
								 courses = courseList)
		else:
			schools = self.DatabaseCursor.Execute('''
				SELECT SchoolId, name
				FROM School
				ORDER BY name''')
			return self.RenderTemplate('Candidate/View/Candidate/Registration/Step-1', schools = schools, course = course)


class QSEView (Handler):
	__method__ = 'GET'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/qse/'

	def Handle(self, *,
			candidate: Candidate):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment')):
			raise Forbidden()

		qse = self.DatabaseCursor.Execute('''
			SELECT *
			FROM CandidateQSE
			WHERE CandidateId = %s
			''',
			(candidate.Id))
		qse = qse[0] if len(qse) > 0 else None

		familyIncome = self.DatabaseCursor.Execute('''
			SELECT * 
			FROM CandidateFamilyIncome
			WHERE CandidateId = %s
			''', (candidate.Id))

		status = validate(qse, familyIncome)
		
		return self.RenderTemplate('Candidate/View/Candidate/QSE/CandidateQSE',

							 candidate = candidate,
							 qse = qse,
							 familyIncome = familyIncome,
							 status = status)

class QSEStep (Handler):
	__method__ = 'GET'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/qse/1/'

	def Handle(self, *,
			candidate: Candidate,
			token: str = None):

		if token != getToken(candidate.Id) and not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment')):
			raise Forbidden()

		qse = self.DatabaseCursor.Execute('''
			SELECT *
			FROM CandidateQSE
			WHERE CandidateId = %s
			''',
			(candidate.Id))
		qse = qse[0] if len(qse) > 0 else None
		
		return self.RenderTemplate('Candidate/View/Candidate/QSE/Step1',
							 candidate = candidate,
							 qse = qse,
							 token = token)

class QSEStep (Handler):
	__method__ = 'GET'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/qse/2/'

	def Handle(self, *,
			candidate: Candidate,
			token: str = None):

		if token != getToken(candidate.Id) and not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment')):
			raise Forbidden()

		qse = self.DatabaseCursor.Execute('''
			SELECT *
			FROM CandidateQSE
			WHERE CandidateId = %s
			''',
			(candidate.Id))
		qse = qse[0] if len(qse) > 0 else None

		familyIncome = self.DatabaseCursor.Execute('''
			SELECT * 
			FROM CandidateFamilyIncome
			WHERE CandidateId = %s
			''', (candidate.Id))
		
		return self.RenderTemplate('Candidate/View/Candidate/QSE/Step2',
							 candidate = candidate,
							 qse = qse,
							 familyIncome = familyIncome, 
							 token = token)



class QSEFamily (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/qse/family/'

	def Handle(self, *,
		candidate: Candidate,
		MemberName: str,
		Relation: str,
		Income: str,
		token: str = None):

		if token != getToken(candidate.Id) and not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment')):
			raise Forbidden()

		self.DatabaseCursor.Execute('''
			INSERT INTO  CandidateFamilyIncome 
			(CandidateId, MemberName, Relation, Income) 
			VALUES (%s, '%s', '%s', %s)
			''' % (candidate.Id, MemberName, Relation, Income.replace(",",".")))
		self.DatabaseCursor.Commit()
		raise NoContent()

class QSEReport (Handler):
	__method__ = 'GET'
	__path__ = '/report/qse/'

	def getCandidateName(self, qse):
		candidate = self.DatabaseCursor.Execute('''
				SELECT *
				FROM Candidate
				WHERE Id = %s
				''',
				(qse.CandidateId))
		if len(candidate) == 0:
			return ""
		return candidate[0].student_name

	def Handle(self):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'Export')):
			raise Forbidden()

		workbook = Workbook(optimized_write = True)
		worksheet = workbook.create_sheet()
		worksheet.title = "Relatório QSE"

		qse_entries = self.DatabaseCursor.Execute('''
			SELECT *
			FROM CandidateQSE
			''')
		max_family_entries = 0
		for qse in qse_entries:
			family_income = self.DatabaseCursor.Execute('''
				SELECT * 
				FROM CandidateFamilyIncome
				WHERE CandidateId = %s
				''', (qse.CandidateId))
			max_family_entries = max(max_family_entries, len(family_income))

		status_columns = [
			("Situação", lambda status: status['status']),
			("Desempenho Acadêmico", lambda status: status.get('academic_performance', "")),
			("Inconsistências", lambda status: status.get('inconsistency', "")),
			("Renda per capta", lambda status: status.get('income_per_capta', "")),
			("Gastos", lambda status: status.get('expenses', ""))
		]

		qse_columns = [
			("Aluno", lambda qse: self.getCandidateName(qse)),
			("Mensalidade Atual", lambda qse: qse.CurrentMonthlyPayment),
			("Mensalidade Pretendida", lambda qse: qse.IntendedMonthlyPayment),
			("Aluno Apogeu", lambda qse: qse.IsFromApogeu),
			("Origem escolar", lambda qse: qse.Schooling),
			("Condição de moradia", lambda qse: qse.Housing),
			("Valor do condomínio", lambda qse: qse.HouseTax),
			("Aluguel ou prestação da casa", lambda qse: qse.Rent),
			("Despesas com segunda moradia", lambda qse: qse.SecondHouseTax),
			("Gastos com educação", lambda qse: qse.EducationalExpenses),
			("Gastos com alimentação", lambda qse: qse.FoodExpenses),
			("Gastos com telefone", lambda qse: qse.PhoneExpenses),
			("Gastos com água", lambda qse: qse.WaterExpenses),
			("Gastos com saúde", lambda qse: qse.HealthExpenses),
			("Gastos com transporte", lambda qse: qse.TransportationExpenses),
			("Gastos com cartão de crédito", lambda qse: qse.CreditCardExpenses),
			("Outros gastos", lambda qse: qse.ExtraExpenses),
			("Desempenho acadêmico", lambda qse: qse.AcademicPerformance)
		]
		family_columns = [
			("Nome do parente", lambda familyMember: familyMember.MemberName),
			("Grau de parentesco", lambda familyMember: familyMember.Relation),
			("Renda", lambda familyMember: familyMember.Income),
		]
		headerRow = []
		for i, (header, field) in enumerate(status_columns):
			headerRow.append(header)
		for i, (header, field) in enumerate(qse_columns):
			headerRow.append(header)
		for i in range(max_family_entries):
			for j, (header, field) in enumerate(family_columns):
				headerRow.append(header + " " + str(i+1))
		worksheet.append(headerRow)

		for qse in qse_entries:
			family_income = self.DatabaseCursor.Execute('''
				SELECT * 
				FROM CandidateFamilyIncome
				WHERE CandidateId = %s
				''', (qse.CandidateId))

			status = validate(qse, family_income)
			row = [field(status) for (header, field) in status_columns]
			row += [field(qse) for (header, field) in qse_columns]

			for family_member in family_income:
				row += [field(family_member) for (header, field) in family_columns]
			worksheet.append(row)

		outputFile = BytesIO()
		workbook.save(outputFile)
		self.Headers.append(('Content-Disposition', 'attachment; filename=qse.xlsx'))
		self.ContentType = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
		return outputFile.getvalue()


class QSE (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/qse/family/=/delete/'

	def Handle(self, *,
		candidate: Candidate,
		token: str = None):

		if token != getToken(candidate.Id) and not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment')):
			raise Forbidden()

		self.DatabaseCursor.Execute('''
			DELETE FROM CandidateFamilyIncome
			WHERE CandidateId = %s''' % candidate.Id)

		self.DatabaseCursor.Commit()
		raise NoContent()

class QSE (Handler):
	__method__ = 'GET'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/qse/success/'

	def Handle(self, *,
		candidate: Candidate, 
		token: str = None):
		return self.RenderTemplate('Candidate/View/Candidate/QSE/Success')
		
class QSE (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/qse/'

	def Handle(self, *,
		candidate: Candidate,
		CurrentMonthlyPayment: str,
		IntendedMonthlyPayment: str,
		IsFromApogeu: str,
		Schooling: str,
		Housing: str,
		HouseTax: str,
		Rent: str,
		SecondHouseTax: str,
		EducationalExpenses: str,
		FoodExpenses: str,
		PhoneExpenses: str,
		WaterExpenses: str,
		HealthExpenses: str,
		TransportationExpenses: str,
		CreditCardExpenses: str,
		ExtraExpenses: str,
		AcademicPerformance: str,
		EnergyExpenses: str,
		IPTU: str,
		Pension: str,
		BolsaFamilia: str,
		fromStudent: str,
		token: str = None):

		if token != getToken(candidate.Id) and not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment')):
			raise Forbidden()
			
		qseFields = [
			'CurrentMonthlyPayment', 
			'IntendedMonthlyPayment',
			'IsFromApogeu',
			'Schooling',
			'Housing',
			'HouseTax',
			'Rent',
			'SecondHouseTax',
			'EducationalExpenses',
			'FoodExpenses',
			'PhoneExpenses',
			'WaterExpenses',
			'HealthExpenses',
			'TransportationExpenses',
			'CreditCardExpenses',
			'ExtraExpenses',
			'AcademicPerformance',
			'EnergyExpenses',
			'IPTU',
			'Pension',
			'BolsaFamilia'
		]
		qseStrFields = [
			'IsFromApogeu',
			'Schooling',
			'Housing'
		]

		query = "INSERT INTO CandidateQSE (CandidateId"
		for field in qseFields:
			query += ", " + field
		query += ") VALUES (%s" % candidate.Id
		for field in qseFields:
			if field in qseStrFields:
				query += ", '" + locals()[field] + "'"
			else:
				query += ", " + (locals()[field]).replace(",",".")
		query += ") ON DUPLICATE KEY UPDATE "
		for field in qseFields:
			query += field + " = VALUES (" + field + "), "
		self.DatabaseCursor.Execute(query[:-2])
		self.DatabaseCursor.Commit()
		
		if fromStudent != "y":
			self.DatabaseCursor.Execute('''
				DELETE FROM CandidateFamilyIncome
				WHERE CandidateId = %s''' % candidate.Id)

			self.DatabaseCursor.Commit()
		
		raise NoContent()

def validate(qse, familyIncome):
	if qse is None or not familyIncome or len(familyIncome) == 0:
		return {"status": "Incompleto"}
	
	totalIncome = qse.Pension + qse.BolsaFamilia
	nRelatives = 0
	for income in familyIncome:
		nRelatives += 1
		totalIncome += income.Income

	incomePerCapta = totalIncome/nRelatives

	totalExpenses = qse.IntendedMonthlyPayment + \
					qse.HouseTax + \
					qse.Rent + \
					qse.SecondHouseTax + \
					qse.EducationalExpenses + \
					qse.FoodExpenses + \
					qse.PhoneExpenses + \
					qse.WaterExpenses + \
					qse.HealthExpenses + \
					qse.TransportationExpenses + \
					qse.ExtraExpenses + \
					qse.EnergyExpenses + \
					qse.IPTU

	# academic performance
	if qse.IsFromApogeu and qse.AcademicPerformance < 60:
		academic_performance = "Indeferido (mínimo esperado: 60, obtido: %d)" % qse.AcademicPerformance
	elif not qse.IsFromApogeu and qse.AcademicPerformance < 70:
		academic_performance = "Indeferido (mínimo esperado: 70, obtido: %d)" % qse.AcademicPerformance
	else:
		academic_performance = "Deferido - Desempenho: " + str(qse.AcademicPerformance) + "/100" 

	# inconsistencies
	if totalExpenses > totalIncome:
		inconsistency = "Indeferido: gastos (R$ %.2f) excedem ganhos (R$ %.2f)" % (totalExpenses, totalIncome)
	else:
		inconsistency = "Deferido. Renda: R$ %.2f, Gastos: R$ %.2f" % (totalIncome, totalExpenses)

	# income per capta
	if incomePerCapta > 826:
		income_per_capta = "Indeferido (máximo esperado: R$ 826.00, calculado: R$ %.2f)" % incomePerCapta
	else:
		income_per_capta = "Deferido. Renda per capta: %.2f" % incomePerCapta

	# expenses
	red_expenses = []
	if totalExpenses < 0.85*totalIncome:
		expenses = "Indeferido (gastos totais inferiores a 85% da renda)"
	else:
		# calculate ideal expenses for some fields
		red_expenses_names = []
		var = 0.4

		dic = {
			'FoodExpenses':            {'name': 'Alimentação', 'ideal': 0.30 * totalIncome},
			'EnergyExpenses':          {'name': 'Energia', 'ideal': 0.08 * totalIncome},
			'PhoneExpenses':           {'name': 'Telefone', 'ideal': 0.06 * totalIncome},
			'WaterExpenses':           {'name': 'Água', 'ideal': 0.03 * totalIncome},
			'HealthExpenses':          {'name': 'Saúde', 'ideal': 0.07 * totalIncome},
			'TransportationExpenses':  {'name': 'Transporte', 'ideal': 0.15 * totalIncome}
		}

		for expense, values in dic.items():
			expense_val = getattr(qse, expense)
			expected = values['ideal']
			max_expected = (1.0 + var) * expected
			min_expected = (1.0 - var) * expected
			if expense_val > max_expected or expense_val < min_expected:
				red_expenses.append(' - %s: Esperado: R$ %.2f, real: R$ %.2f' % (values['name'], expected, expense_val))
				red_expenses_names.append(values['name'])

		
		# more than 3 warnings is not ok.
		if len(red_expenses_names) > 3:
			expenses = "Indeferido (gastos fora da faixa esperada em: " + ", ".join(red_expenses_names) + ")"
		#check phone expenses
		elif qse.PhoneExpenses > 300:
			expenses = "Indeferido por gastos excessivos com telefone - R$ %.2f" % qse.PhoneExpenses
		else:
			expenses = "Deferido (não há gastos excessivos)"

	if academic_performance.startswith("Indeferido") or inconsistency.startswith("Indeferido") or income_per_capta.startswith("Indeferido") or expenses.startswith("Indeferido"):
		status = "Indeferido"
	else:
		status = "Deferido"

	is_from_apogeu = "Sim" if qse.IsFromApogeu else "Não" 
	intended_monthly_payment = qse.IntendedMonthlyPayment
	project_montlhy_payment = 0.1*totalIncome

	return {"status": status, 
			"academic_performance": academic_performance, 
			"inconsistency": inconsistency, 
			"income_per_capta": income_per_capta, 
			"expenses": expenses,
			"red_expenses": red_expenses,
			"is_from_apogeu": is_from_apogeu,
			"intended_monthly_payment": "R$ %.2f" % qse.IntendedMonthlyPayment,
			"project_monthly_payment": "R$ %.2f" % max(0.1*totalIncome, qse.IntendedMonthlyPayment)}

def getToken(id):
	return SHA256.new(str(id+27).encode('utf-8')).hexdigest()[:20]

