import datetime
import urllib.error
import urllib.request
from Campus.Business import CampusManager
from Candidate.Business.CandidateManager import CandidateManager
from Candidate.Entity.Candidate import Candidate
from Ozark.Typing import List
from lawrence import config as config, util
from lawrence.HTTPStatus import NotFound, SeeOther
from lawrence.base import Handler
import lawrence.mail
import lawrence.util

class InitialRegistrationView (Handler):
	__method__ = 'GET'
	__path__ = '/candidate/=/registrationf/'

	def Handle(self, *,
			course: int = None):

		if course is None:
			courseList = self.DatabaseCursor.Execute('''
					SELECT
						Course.id AS Id,
						Course.Name AS Name,
						CourseSegment.Name AS SegmentName
					FROM
						Course
						INNER JOIN CourseSegment
						  ON Course.SegmentId = CourseSegment.id
					WHERE
						Course.id IN (SELECT CourseId FROM SchoolClass)
					ORDER BY CourseSegment.SortOrder, Course.SortOrder, Course.Name''')
			return self.RenderTemplate('Candidate/View/Candidate/Registration/Step-3',
								 courses = courseList)
		else:
			schools = self.DatabaseCursor.Execute('''
				SELECT SchoolId, name
				FROM School
				ORDER BY name''')
			return self.RenderTemplate('Candidate/View/Candidate/Registration/Step-1', schools = schools, course = course)

class InitialRegistrationController (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/=/registrationf/'

	def Handle(self, *,
			student_name: str,
			school: str,
			contact_name: str,
			contact_phone: List[str],
			contact_email: str,
			BirthDate: datetime.date,
			CityName: str,
			course: int):

		self.DatabaseCursor.Execute('''
			INSERT INTO Candidate
				(status, student_name, contact_name, school, BirthDate, CityName,
				 contact_email, RegistrationDateTime, next_call_time)
			VALUES ('cancelled', %s, %s, %s, %s, %s, %s, NOW(), NOW())''',
			(student_name,
			 contact_name,
			 school,
			 BirthDate,
			 CityName,
			 contact_email))
		candidateId = self.DatabaseCursor.ScopeIdentity()
		for contactPhone in contact_phone:
			self.DatabaseCursor.Execute('''
				INSERT INTO CandidatePhone (CandidateId, phone)
				VALUES (%s, %s)''',
				(candidateId, contactPhone))
		# FIXME
		duplicateCandidate = [self.GetManager(CandidateManager).GetById(row.Id) for row in self.DatabaseCursor.Execute('''
			SELECT Candidate.Id
			FROM Candidate
				 INNER JOIN Campus
					 ON Candidate.CampusId = Campus.Id
				 INNER JOIN Course
					 ON Candidate.CourseId = Course.Id
			WHERE status != 'cancelled' AND
				  contact_email = %s''',
			(contact_email, ))]
		self.DatabaseCursor.Commit()
		if duplicateCandidate:
			candidate = Candidate(Id = candidateId, contact_name = contact_name, contact_email = contact_email)
			self.Model.DuplicateCandidateList = duplicateCandidate
			lawrence.mail.Queue(config.candidate_email,
								contact_email,
								self.RenderTemplate('Candidate/View/Candidate/Registration/DuplicateEmail', candidate = candidate, CandidateId = candidate.Id))
			return self.RenderTemplate('Candidate/View/Candidate/Registration/Duplicate', candidate = candidate)
		
		if course is not None:
			self.DatabaseCursor.Execute('''
				UPDATE Candidate
				SET CourseId = %s
				WHERE id = %s''',
				(course,
				 candidateId))
			self.DatabaseCursor.Commit()
			course_db = self.DatabaseCursor.Execute('''
					SELECT
						Course.Id,
						Course.SegmentId
					FROM
						Course
						INNER JOIN CourseSegment
						  ON Course.SegmentId = CourseSegment.id
					WHERE
						Course.Id IN (SELECT CourseId FROM SchoolClass) AND 
						Course.Id = %s AND
						Course.SegmentId = 4''', (course))
			if len(course_db) > 0:
				raise SeeOther('/candidate/%s/registrationf/?alt=y' % (candidateId))

		raise SeeOther('/candidate/%s/registrationf/' % (candidateId))

class RegistrationView (Handler):
	__method__ = 'GET'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/registrationf/'

	def Handle(self, *,
			candidate: Candidate,
			alt: str = 'n',
			enem_error: str = None):

		if alt == "y":
			alt_grades = self.DatabaseCursor.Execute('''
				SELECT CandidateId
				FROM CandidateAlternativeGrade
				WHERE CandidateId = %s''', 
				(candidate.Id, ))
			if len(alt_grades) == 0:
				alt_exams = ['ENEM', 'ITA', 'IME', 'EsPCEx', 'AFA', 'EFOMM', 'EsSA', 'Escola Naval', 'Colégio Naval', 'EPCAr']
				return self.RenderTemplate('Candidate/View/Candidate/Registration/Step-2', candidate = candidate, alt_exams=alt_exams)

		if candidate.Course.Id is None:
			courseList = self.DatabaseCursor.Execute('''
				SELECT
					Course.id AS Id,
					Course.Name AS Name,
					CourseSegment.Name AS SegmentName
				FROM
					Course
					INNER JOIN CourseSegment
					  ON Course.SegmentId = CourseSegment.id
				WHERE
					Course.id IN (SELECT CourseId FROM SchoolClass)
				ORDER BY CourseSegment.SortOrder, Course.SortOrder, Course.Name''')
			return self.RenderTemplate('Candidate/View/Candidate/Registration/Step-3',
								 courses = courseList)

		if candidate.Campus is None:
			validCampusList = self.DatabaseCursor.Execute('''
				SELECT
					Campus.id AS Id,
					Campus.name AS Name
				FROM
					Campus
					INNER JOIN (SELECT DISTINCT CourseId, CampusId FROM SchoolClass) AS T ON T.CampusId = Campus.id
				WHERE
					T.CourseId = %s
				ORDER BY Name''',
				(candidate.Course.Id, ))
			if len(validCampusList) > 1:
				return self.RenderTemplate('Candidate/View/Candidate/Registration/Step-4',
									 ValidCampus = validCampusList)
			if len(validCampusList) == 0:
				raise NotFound()
			campusId = validCampusList[0].Id
			examCampusId = self.DatabaseCursor.Single('''
				SELECT
					COALESCE(ExamCampusMap.ExamCampusId, Campus.Id) AS ExamCampusId
				FROM
					Campus
					LEFT OUTER JOIN ExamCampusMap
					  ON ExamCampusMap.CampusId = Campus.Id
				WHERE
					Campus.Id = %s''',
				(campusId, )).ExamCampusId
			self.DatabaseCursor.Execute('''
				UPDATE Candidate
				SET CampusId = %s,
					ExamCampusId = %s
				WHERE Id = %s''',
				(campusId,
				 examCampusId,
				 candidate.Id))
			self.DatabaseCursor.Commit()
			candidate.ExamCampus = self.GetManager(CampusManager).GetById(examCampusId)
			candidate.Campus = self.GetManager(CampusManager).GetById(campusId)
		if candidate.ExamDateTime is None and not hasattr(candidate, 'enem'):
			examDateTimeList = []
			if candidate.Course.IsExamEligible:
				minimumPriority = self.DatabaseCursor.Single('''
					SELECT MIN(Priority) AS MinimumPriority
					FROM CampusExamDateTime
					WHERE
						CampusId = %s AND
						DATE(ExamDateTime) > TIMESTAMPADD(HOUR, -6, NOW())''', (candidate.ExamCampus.Id, )).MinimumPriority
				if minimumPriority is not None:
					examDateTimeList = [row.ExamDateTime for row in
						self.DatabaseCursor.Execute('''
							SELECT ExamDateTime
							FROM CampusExamDateTime
							WHERE
								CampusId = %s AND
								DATE(ExamDateTime) > TIMESTAMPADD(HOUR, -6, NOW()) AND
								Priority = %s
							ORDER BY ExamDateTime''',
							(candidate.ExamCampus.Id,
							 minimumPriority))]
			if len(examDateTimeList) > 1 or candidate.Course.IsENEMEligible:
				return self.RenderTemplate('Candidate/View/Candidate/Registration/Step-6',
									 candidate = candidate,
									 examDateTimeList = examDateTimeList)
			if len(examDateTimeList) == 0:
				self.DatabaseCursor.Execute('''
					UPDATE Candidate
					SET status = 'unconfirmed'
					WHERE id = %s''',
					(candidate.Id, ))
			if len(examDateTimeList) == 1:
				self.DatabaseCursor.Execute('''
					UPDATE Candidate
					SET ExamDateTime = %s,
						status = 'unconfirmed'
					WHERE id = %s''',
					(examDateTimeList[0],
					 candidate.Id))
				candidate.ExamDateTime = examDateTimeList[0]

				self.DatabaseCursor.Commit()
			else:
				self.DatabaseCursor.Execute('''
					UPDATE Candidate
					SET status = 'present'
					WHERE id = %s''',
					[candidate.Id])
				self.DatabaseCursor.Commit()
		else:
			self.DatabaseCursor.Execute('''
                                        UPDATE Candidate
                                        SET status = 'present'
                                        WHERE id = %s''',
                                        [candidate.Id])
			self.DatabaseCursor.Commit()
		import hashlib
		self.Model.Candidate = candidate
		self.Model.AuthenticationToken = hashlib.sha512(("0eJn7til;" + str(candidate.Id)).encode('utf-8')).hexdigest()
	#	if candidate.ExamDateTime:
		lawrence.mail.Queue(config.candidate_email,
								candidate.contact_email,
								self.RenderTemplate('Candidate/View/Candidate/Registration/ThankYouEmail', candidate = candidate))
		return self.RenderTemplate('Candidate/View/Candidate/Registration/ThankYou', candidate = candidate)


class Registration (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/registrationf/'

	def Handle(self, *,
			candidate: Candidate,

			course: int = None,
			CampusId: int = None,

			ExamDateTime: datetime.datetime = None,

			enem_cpf: str = None,
			enem_password: str = None,

			alternative_grade_name: str = None,
			alternative_grade: str = None):

		# RD Station
		if config.RDStationToken is not None:
			from urllib.parse import urlencode
			import urllib.request, urllib.error, urllib.parse
			URL = 'https://www.rdstation.com.br/api/1.2/conversions'
			requestData = [
				('token_rdstation', config.RDStationToken),
				('identificador', 'inscricao-farabe'),
				('email', candidate.contact_email),
				('nome', candidate.student_name),
			]
			if candidate.Course.Name is not None:
				requestData.append(('cargo', candidate.Course.Name.lower()))
			for contactPhone in candidate.ContactPhones:
				requestData.append(('telefone', util.phone(contactPhone)))
			requestData = [(k.encode('utf-8'), v.encode('utf-8')) for k, v in requestData]
			try:
				request = urllib.request.Request(URL, urlencode(requestData).encode('utf-8'))
				urllib.request.urlopen(request)
			except urllib.error.HTTPError:
				pass


		if alternative_grade_name is not None:
			self.DatabaseCursor.Execute('''
				INSERT INTO CandidateAlternativeGrade 
 				(CandidateId, ExamName, Grade) 
 				VALUES (%s, %s, %s)
				''',
				(candidate.Id,
				 alternative_grade_name, alternative_grade))
			self.DatabaseCursor.Commit()

		elif CampusId is not None:
			examCampusId = self.DatabaseCursor.Single('''
				SELECT
					COALESCE(ExamCampusMap.ExamCampusId, Campus.Id) AS ExamCampusId
				FROM
					Campus
					LEFT OUTER JOIN ExamCampusMap
					  ON ExamCampusMap.CampusId = Campus.Id
				WHERE
					Campus.Id = %s''',
				(CampusId, )).ExamCampusId
			self.DatabaseCursor.Execute('''
				UPDATE
					Candidate
				SET
					CampusId = %s,
					ExamCampusId = %s
				WHERE
					id = %s''',
				(CampusId,
				 examCampusId,
				 candidate.Id))
			# examDateTimeList = self.DatabaseCursor.Execute('''
			# 	SELECT ExamDateTime
			# 	FROM course_datetimes
			# 	WHERE CourseId = %s AND
			# 		  DATE(ExamDateTime) > TIMESTAMPADD(HOUR, -6, NOW())
			# 	ORDER BY ExamDateTime''',
			# 	(candidate.Course.Id, ))
			# examDateTimeList = [row.ExamDateTime for row in examDateTimeList]
			# # Possibly skip the exam time choice.
			# if not examDateTimeList:
			# 	self.DatabaseCursor.Execute('''
			# 		UPDATE Candidate
			# 		SET status = 'unconfirmed'
			# 		WHERE id = %s''',
			# 		(candidate.Id, ))
			self.DatabaseCursor.Commit()
		elif ExamDateTime is not None:
			self.DatabaseCursor.Execute('''
				UPDATE Candidate
				SET ExamDateTime = %s,
					status = 'unconfirmed'
				WHERE id = %s''',
				(ExamDateTime,
				 candidate.Id))
			self.DatabaseCursor.Commit()
		elif enem_cpf is not None:
			try:
				candidate.FetchEnemResult(enem_cpf, enem_password)
			except Candidate.InvalidLogin:
				raise SeeOther('/candidate/%s/registrationf/' % candidate.Id,
					params = {'enem_error': 'invalid_login'})
			except Candidate.NetworkError:
				raise SeeOther('/candidate/%s/registrationf/' % candidate.Id,
					params = {'enem_error': 'network_error'})
		raise SeeOther('/candidate/%s/registrationf/' % candidate.Id)

class NextExam (Handler):
	__method__ = 'GET'
	__path__ = '/Candidate/=/NextExam/'

	def Handle(self):
		raise NotImplementedError()
		# try:
		# 	self.Model.NextExamDate = self.DatabaseCursor.Single('''
		# 		SELECT MIN(DATE(ExamDateTime)) AS ExamDate
		# 		FROM course_datetimes
		# 		WHERE DATE(ExamDateTime) > TIMESTAMPADD(HOUR, -6, NOW())
		# 		ORDER BY ExamDateTime''').ExamDate
		# 	self.Model.NextExamTimeList = [row.ExamTime for row in self.DatabaseCursor.Execute('''
		# 		SELECT DISTINCT TIME(ExamDateTime) AS ExamTime
		# 		FROM course_datetimes
		# 		WHERE DATE(ExamDateTime) = %s
		# 		ORDER BY ExamDateTime''', (self.Model.NextExamDate, ))]
		# 	import json
		# 	import lawrence.util
		# 	self.Headers.append(('Access-Control-Allow-Origin', '*'))
		# 	return json.dumps({'Date': lawrence.util.ShortDate(self.Model.NextExamDate),
		# 					   'TimeList': [lawrence.util.ShortTime(time) for time in self.Model.NextExamTimeList]})
		# except ValueError:
		# 	raise NotFound()

class EmailConfirmView (Handler):
	__method__ = 'GET'
	__path__ = '/Candidate/(?P<candidateId>[1-9][0-9]*)/ConfirmEmail/'

	def Handle(self, *,
			candidate: Candidate,
			AuthenticationToken: str):
		if candidate.status != 'unconfirmed':
			raise NotFound()
		import hashlib
		if AuthenticationToken != hashlib.sha512("0eJn7til;" + str(candidate.Id)).hexdigest():
			raise NotFound

		self.DatabaseCursor.Execute('UPDATE Candidate SET status = %s WHERE Id = %s',
			('confirmed', candidate.Id))

		self.DatabaseCursor.Execute('''
			INSERT INTO audit_candidate_status (CandidateId, OldValue, NewValue, UserId)
			VALUES (%s, %s, %s, %s)''',
			(candidate.Id, 'unconfirmed', 'confirmed', None))

		self.DatabaseCursor.Commit()

		return self.RenderTemplate('Candidate/View/Candidate/Registration/EmailConfirmed')

class FixPhoneView (Handler):
	__method__ = 'GET'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/fix-phone/'

	def Handle(self, *,
			candidate: Candidate):
		if candidate.status != 'cancelled':
			raise NotFound()
		return self.RenderTemplate('Candidate/View/Candidate/Registration/FixPhone',
							 candidate = candidate)


class FixPhone (Handler):
	__method__ = 'POST'
	__path__ = '/candidate/(?P<candidateId>[1-9][0-9]*)/fix-phone/'

	def Handle(self, *,
			candidate: Candidate,
			contact_phone: List[str]):
		if candidate.status != 'cancelled':
			raise NotFound()
		self.DatabaseCursor.begin()
		self.DatabaseCursor.Execute('''
			UPDATE Candidate
			SET status = 'unconfirmed'
			WHERE id = %s''',
			(candidate.Id, ))
		self.DatabaseCursor.Execute('''
			DELETE FROM CandidatePhone
			WHERE CandidateId = %s''',
			(candidate.Id, ))
		for contactPhone in contact_phone:
			self.DatabaseCursor.Execute('''
				INSERT INTO CandidatePhone
				VALUES (%s, %s)''',
				(candidate.Id,
				 contactPhone))
		self.DatabaseCursor.Commit()
		raise SeeOther('/candidate/%s/registrationf/' % candidate.Id)
