from io import BytesIO
from Tools.EntityFromXlsx import ReadWorksheet, Entity
from openpyxl import load_workbook
from pymysql import IntegrityError
from User.Business.UserPermissionManager import UserPermissionManager
from User.Entity.UserPermission import UserPermission
from lawrence import util as util
from lawrence.HTTPStatus import Forbidden
from lawrence.base import Handler

class ImportTags (Handler):
	__method__ = 'POST'
	__path__ = '/Candidate/ImportTags/'

	@classmethod
	def NormalizeForMatch(cls, s):
		return s.lower()  # FIXME

	@classmethod
	def MatchesName(cls, s, t):
		return (" " + cls.NormalizeForMatch(s) + " ") in (" " + cls.NormalizeForMatch(t) + " ")

	@classmethod
	def BestMatch(cls, needle, haystack, key = lambda entity: entity.Name):
		matchingEntityList = [entity for entity in haystack if cls.MatchesName(needle, key(entity))]
		if len(matchingEntityList) == 1:
			return matchingEntityList[0]
		elif len(matchingEntityList) == 0:
			raise IndexError('match not found')
		else:
			raise IndexError('more than one match found: ' + str(matchingEntityList) )

	def ImportTags(self, worksheet):
		for entity in ReadWorksheet(worksheet, {
			'InscricaoFarabe': str,
			'Marcador': str,
		}):
			try:
				matchingTag = self.BestMatch(entity.Marcador, self.TagList)
			except IndexError as ex:
				if str(ex) != 'match not found':
					raise
				self.DatabaseCursor.Begin()
				self.DatabaseCursor.Execute('''
					INSERT INTO CandidateTag (Name)
					VALUES (%s)''',
					(entity.Marcador, ))
				self.TagList = self.DatabaseCursor.Execute('''
					SELECT
						id AS Id,
						name AS Name
					FROM CandidateTag''')
				self.DatabaseCursor.Commit()

	def ImportStudents(self, worksheet):
		for entity in ReadWorksheet(worksheet, {
			'InscricaoFarabe': str,
			'Marcador': str,
		}):
			matchingCandidate = Entity(Id = util.DecodeId(int(entity.InscricaoFarabe)))
			matchingTag = self.BestMatch(entity.Marcador, self.TagList)
			
			CandidateList = None
			
			self.DatabaseCursor.Begin()
			CandidateList = self.DatabaseCursor.Execute('''
				 SELECT * FROM CandidateTagMap WHERE CandidateId = %s''', matchingCandidate.Id)
			
			if CandidateList:
				self.DatabaseCursor.Begin()
				try:
					self.DatabaseCursor.Execute('''
						UPDATE CandidateTagMap
						SET TagId = %s
						WHERE CandidateId = %s''',
						(matchingTag.Id, matchingCandidate.Id))
				except IntegrityError:
					pass
				self.DatabaseCursor.Commit()
			else:
				self.DatabaseCursor.Begin()
				try:
					self.DatabaseCursor.Execute('''
						INSERT INTO CandidateTagMap (CandidateId, TagId)
						VALUES (%s, %s)''',
						(matchingCandidate.Id, matchingTag.Id))
				except IntegrityError:
					pass
				self.DatabaseCursor.Commit()

	def Handle(self,
			TagSpreadsheet: bytes):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ImportTags')):
			raise Forbidden()

		self.TagList = self.DatabaseCursor.Execute('''
			SELECT
				id AS Id,
				name AS Name
			FROM CandidateTag''')

		tagSpreadsheetFile = BytesIO(TagSpreadsheet)
		workbook = load_workbook(tagSpreadsheetFile)
		worksheet = workbook.get_sheet_by_name(name = "Marcadores")
		self.ImportTags(worksheet)
		self.ImportStudents(worksheet)

		return self.RenderTemplate('Candidate/View/Candidate/ImportTags')

