<%inherit file="/View/_Layout.mako" />
<%namespace file="/View/_Editable.mako" import="editable" />
<%namespace file="/Candidate/View/Candidate/_Fields.mako" import="candidate_course, candidate_activities" />
<%include file="/View/_Spinner.mako" />

<h1>${editable('contact_name',
               candidate.contact_name,
               candidate.path,
               disabled = not Model.CanEdit)}</h1>

<p class="lead">
% for contact_phone in candidate.ContactPhones:
    ${u"/" if contact_phone is not candidate.ContactPhones[0] else u""}
    ${contact_phone | phone}
% endfor
</p>

<form class="form-horizontal">

<div class="row-fluid">

<div class="span6">
    <h3 class="box-header">Dados do aluno</h3>

    <div class="box">
        <div class="control-group">
            <label class="control-label" for="student-name">
                Nome</label>
            <div class="controls">
                ${editable('student_name',
                           candidate.student_name,
                           candidate.path,
                           disabled = not Model.CanEdit)}
            </div>
        </div>

        <div class="control-group">
            <label class="control-label" for="candidate-school">
                Colégio atual</label>
            <div class="controls">
                ${editable('school',
                           candidate.school,
                           candidate.path,
                           type = 'typeahead',
                           source = '/school/?json=typeahead',
                           disabled = not Model.CanEdit)}
            </div>
        </div>
    </div>
</div>

<div class="span6">
    <h3 class="box-header">Detalhes da inscrição</h3>

    <div class="box">
        <div class="control-group">
            <label class="control-label" for="candidate-campus">
                Unidade</label>
            <div class="controls">
                ${editable('campus',
                           str(candidate.Campus.Id),
                           candidate.path,
                           type = 'select',
                           source = candidate.path + 'valid-Campus/',
                           display = candidate.Campus.Name,
                           disabled = not Model.CanEdit)}
            </div>
        </div>

        <div class="control-group">
            <label class="control-label" for="candidate-course">
                Campanha</label>
            <div class="controls">
                ${candidate_course(candidate)}
            </div>
        </div>

        <div class="control-group">
            <label class="control-label" for="candidate-exam-campus">
                Unidade do exame</label>
            <div class="controls">
				${editable('exam_campus',
                           str(candidate.ExamCampus.Id),
                           candidate.path,
                           display = candidate.ExamCampus.Name,
                           type = 'select',
                           source = '/campus/?ajax=1',
                           disabled = not Model.CanEdit)}
            </div>
        </div>

        <div class="control-group">
            <label class="control-label" for="candidate-ExamDateTime">
                Data/hora do exame</label>
            <div class="controls">
				${editable('ExamDateTime',
						   candidate.ExamDateTime,
						   candidate.path,
						   display = LongDateTime,
						   type = 'select',
						   source = candidate.path + 'valid-exam-datetimes/',
                           disabled = not Model.CanEdit)}
            </div>
        </div>
    </div>
</div>

<div class="row-fluid">
	<div class="offset3 span6">
		${candidate_activities(candidate, showAppointmentButton = False)}
	</div>
</div>

<div class="row-fluid">
	<div class="offset3 span6">
		<%namespace file="/Candidate/View/Candidate/_Negotiations.mako" import="candidate_negotiations" />
		${candidate_negotiations(candidate)}
	</div>
</div>

</div>

<div class="row-fluid">
    % if candidateQueue.Filter.Status != 'cancelled' and Model.CanEdit and Model.CanCancel:
        ${cancel()}
    % endif
    % if candidateQueue.Filter.Status != 'dropped' and Model.CanEdit:
        ${drop()}
    % endif

        ${postpone()}

    % if len(candidateQueue.Filter.Status) == 1 and candidateQueue.Filter.Status[0] in ('unconfirmed', 'confirmed', 'absent', 'failed') and Model.CanEdit:
        ${reschedule_exam()}
    % endif
    % if len(candidateQueue.Filter.Status) == 1 and candidateQueue.Filter.Status[0] == 'unconfirmed' and Model.CanEdit:
        ${confirm()}
    % endif

	% if Model.CanScheduleAppointment:
        ${appointment()}
	% endif
</div>

</form>

<%def name="postpone()">
<a id="postpone-call"
   class="btn quick-btn span2">
	<i class="icon-phone"></i>
	<span class="quick-btn-title">Ligar mais tarde&hellip;</span>
</a>
<div style="display: none" id="postpone-call-popover">
	<div class="postpone-call-date"></div>
	<hr>
	<div class="row-fluid">
		<a class="btn span6 postpone-call-button" data-hours="1" style="margin-bottom: 8px;">1 hora</a>
		<a class="btn span6 postpone-call-button" data-hours="4" style="margin-bottom: 8px;">4 horas</a>
	</div>
</div>
<script>
$('#postpone-call').popover({
	html: true,
	content: $('#postpone-call-popover').html(),
	placement: 'top',
});

$('#postpone-call').on('shown', function () {
	var self = this;

	$('.postpone-call-date').datepicker({
		value: '+0d',
		startDate: '+1d',
	}).on('changeDate', function (e) {
		var today = new Date();
		var hours = Math.round((e.date - today) / (60 * 60 * 1000)) + 6;
		var $content = $(self).data('popover').$tip.find('.popover-content');
		var $loading = $('<div class="editableform-loading"></div>');
		$loading.width($content.width());
		$loading.height($content.height());
		$content.empty().append($loading);
		
		$.post('/candidate/${candidate.Id | n,js}/postpone/',
			   {hours: hours},
			   function () { PostRedirect('/Candidate/Queue/${candidateQueue.Id | js}/Front/'); });
	});
	$('#postpone-call').popover('updatePlacement');
});

$('body').on('click', '.postpone-call-button', function (e) {
	$.post('/candidate/${candidate.Id | n,js}/postpone/',
		   {hours: this.dataset.hours},
		   function () { PostRedirect('/Candidate/Queue/${candidateQueue.Id | js}/Front/'); });

	e.stopPropagation();
	e.preventDefault();
	return false;
});
</script>
</%def>

<%def name="cancel()">
<a id="cancel-candidate"
   class="btn quick-btn span2 btn-red"
   title="O cancelamento de uma inscrição é definitivo!">
	<i class="icon-trash"></i>
	<span class="quick-btn-title">Cancelar inscrição</span>
</a>
<script>
$('#cancel-candidate').click(function () {
    $.post('/candidate/${candidate.Id | n,js}/',
           {name: 'status', value: 'cancelled'},
           function () { PostRedirect('/Candidate/Queue/${candidateQueue.Id | js}/Front/'); });
}).tooltip();
</script>
</%def>

<%def name="drop()">
<a id="drop-candidate"
   class="btn quick-btn span2">
	<i class="icon-remove"></i>
	<span class="quick-btn-title">Marcar como desistente</span>
</a>
<script>
$('#drop-candidate').click(function () {
    $.post('/candidate/${candidate.Id | n,js}/',
           {name: 'status', value: 'dropped'},
           function () { PostRedirect('/Candidate/Queue/${candidateQueue.Id | js}/Front/'); });
}).tooltip();
</script>
</%def>

<%def name="confirm()">
<a id="confirm-candidate"
   class="btn quick-btn span2 btn-blue">
	<i class="icon-check"></i>
	<span class="quick-btn-title">Confirmar inscrição</span>
</a>
<script>
$('#confirm-candidate').click(function () {
    $.post('/candidate/${candidate.Id | n,js}/',
           {name: 'status', value: 'confirmed'},
           function () { PostRedirect('/Candidate/Queue/${candidateQueue.Id | js}/Front/'); });
});
</script>
</%def>

<%def name="reschedule_exam()">
<%
cursor = db.Cursor()
minimumPriority = cursor.Single('''
    SELECT MIN(Priority) AS MinimumPriority
    FROM CampusExamDateTime
    WHERE
        CampusId = %s AND
        DATE(ExamDateTime) > TIMESTAMPADD(HOUR, -6, NOW())''', (candidate.ExamCampus.Id, )).MinimumPriority
if minimumPriority is not None:
    nextExamDateTimeList = [row.ExamDateTime for row in
        cursor.Execute('''
            SELECT ExamDateTime
            FROM CampusExamDateTime
            WHERE
                CampusId = %s AND
                DATE(ExamDateTime) > TIMESTAMPADD(HOUR, -6, NOW()) AND
                Priority = %s
            ORDER BY ExamDateTime''',
            (candidate.ExamCampus.Id,
             minimumPriority))]
else:
    nextExamDateTimeList = []
%>
<a id="candidate-reschedule-exam"
   class="btn quick-btn span2">
	<i class="icon-mail-forward"></i>
	<span class="quick-btn-title">Alterar data do exame&hellip;</span>
</a>
<div style="display: none" id="candidate-reschedule-exam-popover">
	<div style="height: 106px; overflow-y: scroll;">
% for ExamDateTime in nextExamDateTimeList:
	<a class="btn input-block-level reschedule-exam-datetime"
	   data-exam-datetime="${ExamDateTime | h}"
	   style="margin-bottom: 8px;">
		${ExamDateTime | n,LongDay} de
		${ExamDateTime | n,LongMonth},
		${ExamDateTime | n,time}</a>
% endfor
	</div>
</div>

<script>
$('#candidate-reschedule-exam').popover({
    html: true,
    content: $('#candidate-reschedule-exam-popover').html(),
});

$('body').on('click', '.reschedule-exam-datetime', function () {
    $.post('/candidate/${candidate.Id | n,js}/',
           {name: 'ExamDateTime', value: this.dataset.examDatetime},
           function () {
    $.post('/candidate/${candidate.Id | n,js}/',
           {name: 'status', value: 'unconfirmed'},
           function () { PostRedirect('/Candidate/Queue/${candidateQueue.Id | js}/Front/'); });
    });
});
</script>
</%def>

<%def name="appointment()">
<% import datetime %>

<a id="candidate-appointment-button" class="btn quick-btn span2 btn-blue">
	<i class="icon-calendar"></i>
	<span class="quick-btn-title">Agendar atendimento&hellip;</span>
</a>

<div id="candidate-appointment-popover" style="display: none">
	<div class="row-fluid span12" style="width: 400px !important">
		<div class="span7" style="border-right: 1px #EEE solid">
			<div class="candidate-appointment-date"></div>
		</div>
		<div class="span5">
			<div class="candidate-appointment-time">
			</div>
		</div>
	</div>
	<br clear="both">
</div>

<style>
.timepicker td {
	width: 45px;
	padding: 4px 5px;
	border-radius: 4px;
	cursor: pointer;
	text-align: center;
}

.timepicker td > span {
	display: block;
	width: auto;
	margin: -4px -5px;
	padding: 4px 5px;
}

.timepicker td.highlight > span {
	padding: 3px 4px;
	border: 1px rgb(238, 161, 35) solid;
	border-radius: 4px;
}

.timepicker td.highlight.active > span {
	padding: 4px 5px;
	border: none;
}

.timepicker td:hover {
	background-color: #EEE;
}

.timepicker td.active {
	background-color: rgb(54, 144, 230);
	color: #FFF;
}

.timepicker td.disabled {
	background-color: transparent;
	color: #EEE;
}

.timepicker td.disabled:hover {
	color: #999;
	cursor: default;
}

#candidate-appointment-button + .popover {
	max-width: none;
}
</style>

<script>
$('#candidate-appointment-button').popover({
    html: true,
    content: $('#candidate-appointment-popover').html(),
    placement: 'top',
});

function pad(n) { return n < 10 ? '0' + n : n; };
/**
 * @return {string}
 */
function ISOString(d) {
    return d.getUTCFullYear() + '-' +
           pad(d.getUTCMonth() + 1) + '-' +
           pad(d.getUTCDate());
}

function updateTimePicker() {
	var $loading = $('<div class="editableform-loading"></div>');
	$loading.width($('.candidate-appointment-time').width());
	$loading.height($('.candidate-appointment-time').height());
	$('.candidate-appointment-time').empty().append($loading);
	$.get('${candidate.path | js}appointment/=/valid-time/',
		  {date: ISOString($(this).datepicker('getDate'))},
		  function (data) {
			$('.candidate-appointment-time').html(data);
			$('.timepicker td').click(function () {
				if ($(this).hasClass('disabled')) {
					return false;
				}

				$(this).closest('.timepicker').find('td').removeClass('active');
				$(this).addClass('active');

				$.post('/candidate/${candidate.Id | js}/appointment/=/new/',
					   {date: ISOString($('.popover-content .candidate-appointment-date').datepicker('getDate')),
						time: this.dataset.value},
					   function (data) {
						   $('.widget-users-activity .widget-actions').before(data);
						   $('#candidate-appointment-button').popover('hide');
						   PostRedirect('/Candidate/Queue/${candidateQueue.Id | js}/Front/');
						});
			});
			$('.timepicker td.highlight span').tooltip();
			$('#candidate-appointment-button').popover('updatePlacement');
		  });
}

$('#candidate-appointment-button').on('shown', function () {
    $('.popover-content .candidate-appointment-date').datepicker({
		value: '+1d',
		dateFormat: 'd/M',
		daysOfWeekDisabled: [0],
		startDate: new Date(),
	}).on('changeDate', updateTimePicker);

	updateTimePicker();

	$('#candidate-appointment-button').popover('updatePlacement');
});
</script>
</%def>