# coding=utf-8
<%def name="search_filters()">
<%
from Campus.Business import CampusManager
campusManager = CampusManager(currentUser = user, databaseCursor = db.Cursor())
from collections import namedtuple
from Candidate.Entity.Candidate import Candidate

Option = namedtuple('Option', ['value', 'name'])

CourseList = db.Execute('''
SELECT Course.Id AS value,
       Course.Name AS name
FROM Course''')

StatusList = [Option(*t) for t in Candidate.STATUSES.items()]
AppointmentStatusList = [Option(*t) for t in Candidate.APPOINTMENT_STATUSES.items()]

CampusList = db.Execute('''
SELECT Campus.Id AS value,
       Campus.Name AS name
FROM Campus
WHERE Campus.Id IN %s
ORDER BY name''',
	(campusManager.GetAuthorized() + [0, 0], ))

ExamDateList = [Option(value = row.exam_date, name = LongDay(row.exam_date) + u" de " + LongMonth(row.exam_date)) for row in db.Execute('''
SELECT DISTINCT DATE(ExamDateTime) AS exam_date
FROM Candidate
ORDER BY exam_date''') if row.exam_date is not None]

ExamTimeList = [Option(value = '0' * (8 - len(str(row.exam_time))) + str(row.exam_time),
                       name = ShortTime(row.exam_time)) for row in db.Execute('''
SELECT DISTINCT time(ExamDateTime) AS exam_time
FROM Candidate
ORDER BY exam_time''') if row.exam_time is not None]

SortOrderList = [
	Option(value = 'student_name', name = u"nome"),
	Option(value = 'last_comment', name = u"último comentário"),
	Option(value = 'total_grade', name = u"nota no exame"),
	Option(value = 'enem', name = u"nota no ENEM"),
	Option(value = 'negotiation_gap', name = u"gap da negociação"),
	Option(value = 'rating', name = u"avaliação"),
]

TagList = db.Execute('''
SELECT CandidateTag.Id AS value,
       CandidateTag.Name AS name
FROM CandidateTag
ORDER BY name''')

%>
    <form class="form-search">
        <div class="input-append">
            <input type="text" id="Query"
                   value="${getattr(FormData, 'query', '') | h}"
                   class="search-query input-block-level">
            <button type="button" class="btn" id="QueryButton"><i class="icon-search"></i></button>
        </div>
    </form>

    <% Filter('Course', CourseList, u"Campanha") %>

% if len(CampusList) > 1:
    <% Filter('Campus', CampusList, u"Unidade") %>
% endif

    <% Filter('Status', StatusList, u"Situação") %>

	<div id="AppointmentStatusGroup" style="display: none">
        <% Filter('AppointmentStatus', AppointmentStatusList, u"Situação do atendimento") %>
	</div>

    <% Filter('ExamDate', ExamDateList, u"Dia do exame") %>

    <% Filter('ExamTime', ExamTimeList, u"Hora do exame") %>

    <% Filter('Tag', TagList, u"Marcador") %>

    <% Sort('SortOrder', SortOrderList, u"Ordenar por…") %>

    <h3 class="box-header">Página</h3>
    <div class="box">
		<div data-filter="Page">
			<div class="PagePrevious muted"><i class="icon-caret-left"></i></div><div class="PageNumber">1</div><div class="PageNext"><i class="icon-caret-right"></i></div>
			<input type="hidden" name="Page" value="1">
		</div>
	</div>

    <script src="/static/CandidateIndex.js"></script>

	<hr>

% if Model.CanExport:
    <h2 class="box-header">Exportar lista...</h2>
    <div class="box box-collapsed" id="ExportBox">
		<a href="/candidate/=/export/xlsx/raw/" class="btn quick-btn btn-green btn-block">
			<i class="icon-download-alt"></i>
			<span class="quick-btn-title">Dados de contato</span>
		</a>
		<a href="/candidate/=/export/docx/certificate/" class="btn quick-btn btn-blue btn-block">
			<i class="icon-download-alt"></i>
			<span class="quick-btn-title">Certificados de aprovação</span>
		</a>
% if config.db_name == 'apogeu':
		<a href="/candidate/=/export/docx/certificate-failed/" class="btn quick-btn btn-blue btn-block">
			<i class="icon-download-alt"></i>
			<span class="quick-btn-title">Certificados de reprovação</span>
		</a>
% endif
    </div>
	<script>
	$('#ExportBox a').click(function (e) {
		var filterDict = getFilterDict();
		this.href = this.href.split('?')[0];
		this.href += '?' + queryStringFromObject(filterDict);
	});
	</script>
% endif

% if Model.CanEMail:
	<a class="btn quick-btn btn-block offset3 span6" data-toggle="modal" data-target="#CandidateEmailModal"
       style="margin-bottom: 8px">
		<i class="icon-envelope"></i>
		<span class="quick-btn-title">Enviar email...</span>
	</a>

    <div class="modal fade hide" id="CandidateEmailModal">
        <div class="modal-header">
            <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>
            <h3>Enviar email&hellip;</h3>
        </div>
        <div class="modal-body">
			<form class="form-horizontal" action="/candidate/=/email/" method="POST" id="CandidateEmailForm">
			<div class="control-group">
				<label class="control-label">Assunto</label>
				<div class="controls">
					<input type="text" name="Email.Subject">
				</div>
			</div>

			<div class="control-group">
				<label class="control-label">Conteúdo</label>
				<div class="controls">
					<textarea name="Email.Body"></textarea>
				</div>
			</div>
			</form>
        </div>
        <div class="modal-footer">
            <button type="button" class="btn btn-primary" id="CandidateEmailSubmit">Enviar</button>
        </div>
    </div>

    <div class="modal fade hide" id="CandidateEmailProgressModal">
        <div class="modal-header">
            <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>
            <h3>Enviando emails&hellip;</h3>
        </div>
        <div class="modal-body">
			<div class="progress progress-striped active"><div class="bar" style="width: 100%"></div></div>
        </div>

        <div class="modal-footer">
            <button type="button" class="btn" data-dismiss="modal">OK</button>
        </div>
    </div>

	<script>
	document.getElementById('CandidateEmailSubmit').addEventListener('click', function(e) {
		var candidateEmailForm = document.getElementById('CandidateEmailForm');

		var requestData = { 'Email.Subject': $('[name="Email.Subject"]').val(), 'Email.Body': $('[name="Email.Body"]').val() };
		var filterDict = getFilterDict();
		for (filterName in filterDict) {
			if (!filterDict.hasOwnProperty(filterName)) { continue; }
			var filterValue = filterDict[filterName];
			if (!filterValue) { continue; }
			requestData['CandidateFilter.' + filterName] = filterValue;
		}

		$('#CandidateEmailModal').modal('hide');
		$('#CandidateEmailProgressModal').modal('show');
		$.ajax({
			type: 'POST',
			url: '/candidate/=/email/',
			data: requestData,
		}).success(function (data, textStatus, jqXHR) {
			$('#CandidateEmailProgressModal').find('.modal-body').html('<p>Os emails foram enviados com sucesso.</p>');
			$('#CandidateEmailProgressModal').find('h3').html('Emails enviados');
		});
	});
	</script>
% endif
	
% if Model.CanCreateQueue:
	<a class="btn quick-btn btn-block offset3 span6" data-toggle="modal" data-target="#CandidateQueueNewModal"
       style="margin-bottom: 8px">
		<i class="icon-list"></i>
		<span class="quick-btn-title">Criar fila&hellip;</span>
	</a>

    <div class="modal fade hide" id="CandidateQueueNewModal">
        <div class="modal-header">
            <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>
            <h3>Criar fila&hellip;</h3>
        </div>
        <div class="modal-body">
			<form class="form-horizontal" id="CandidateQueueNewForm">
			<div class="control-group">
				<label class="control-label">Nome</label>
				<div class="controls">
					<input type="text" name="Name">
				</div>
			</div>
			</form>
        </div>
        <div class="modal-footer">
            <button type="button" class="btn btn-primary" id="CandidateQueueNewSubmit">Enviar</button>
        </div>
    </div>

	<script>
    document.getElementById('CandidateQueueNewForm').addEventListener('submit', function(e) {
        // FIXME: fake button click
        e.preventDefault();
    });
	document.getElementById('CandidateQueueNewSubmit').addEventListener('click', function(e) {
		var candidateQueueNewForm = document.getElementById('CandidateQueueNewForm');

		var requestData = { 'Name': $(candidateQueueNewForm).find('[name="Name"]').val() };
		var filterDict = getFilterDict();
		for (filterName in filterDict) {
			if (!filterDict.hasOwnProperty(filterName) || !filterDict[filterName]) { continue; }
			requestData[filterName] = filterDict[filterName];
		}

        PostRedirect('/Candidate/Queue/New/', requestData);

        e.preventDefault();
	});
	</script>
% endif

% if True:
	<a class="btn quick-btn btn-block offset3 span6" id="CandidateReportButton"
       style="margin-bottom: 8px">
		<i class="icon-signal"></i>
		<span class="quick-btn-title">Gerar relatório</span>
	</a>

	<script>
    function updateReport(filterDict) {
		var filterDict = filterDict || getFilterDict();
		var filterQueryString = queryStringFromObject(filterDict);

		var $spinner = CreateSpinner($(document.getElementById('CandidateListing')));

        filterDict.Page = undefined;

		$.ajax({
			url: '/Candidate/_Report/',
			type: 'GET',
			data: filterDict,
		}).success(function (data, textStatus, jqXHR) {
			$(document.getElementById('CandidateListing')).html(data);
			DestroySpinner($spinner);
			window.history.replaceState(filterDict, '', window.location.origin + window.location.pathname + '?' + filterQueryString);
		});
    }
    document.getElementById('CandidateReportButton').addEventListener('click', function (e) {
        $('h3[data-filter]').each(function () {
            $(this).attr('draggable', true);
            this.addEventListener('dragstart', function (e) {
                e.dataTransfer.setData('farabe/candidate-filter', this.dataset.filter);
            }, false);
        });

        updateReport();
    }, false);
	</script>
% endif

% if Model.CanImportTags:
	<a class="btn quick-btn btn-block offset3 span6" id="btnImportTags" style="margin-bottom: 8px">
		<i class="icon-tags"></i>
		<span class="quick-btn-title">Importar marcadores…</span>
	</a>

    <div class="modal fade hide" id="frmImportTags">
        <div class="modal-header">
            <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>
            <h3>Importar marcadores…</h3>
        </div>
        <div class="modal-body">
			<form class="form-horizontal" action="/Candidate/ImportTags/" enctype="multipart/form-data" method="POST">
			<div class="control-group">
				<label class="control-label">Planilha</label>
				<div class="controls">
					<input type="file" name="TagSpreadsheet">
				</div>
			</div>
			</form>
            <p>(<a href="/static/ModeloMarcadores.xlsx">modelo de planilha</a>)</p>
        </div>
        <div class="modal-footer">
            <button type="button" class="btn btn-primary" id="btnSubmitTags">Enviar</button>
        </div>
    </div>

	<script>
    document.getElementById('btnImportTags').addEventListener('click', function (e) {
        $(document.getElementById('frmImportTags')).modal('show');
    });
    document.getElementById('btnSubmitTags').addEventListener('click', function (e) {
        $(document.getElementById('frmImportTags')).find('form').submit();
    })
	</script>
% endif
</%def>

<%def name="filter_styles()">
<style><%include file="CandidateIndex.css" /></style>
</%def>

<%def name="Filter(filterName, optionList, filterLabel)">
<h3 data-filter="${filterName | h}"><span class="FilterLabel">${filterLabel | h}</span><span class="FilterSelection"></span></h3>
<div class="box box-collapsed">
	<a class="btn btnToggleMultiSelect" style="margin-bottom: 8px"><i class="icon-check"></i></a>

    <table data-filter="${filterName | h}"><thead>
    </thead><tbody>
        % for option in optionList:
        <tr>
            <td><input type="radio" name="${filterName | h}" value="${option.value | h}"></td>
            <td>${option.name | h}</td>
            <td><span class="badge"></span></td>
        </tr>
        % endfor
    </tbody></table>
</div>
</%def>

<%def name="Sort(sortName, optionList, sortLabel)">
<h3 class="box-header">${sortLabel | h}</h3>
<div class="box box-collapsed">
    <table data-filter="${sortName | h}">
        % for option in optionList:
        <tr>
            <td><input type="radio" name="${sortName | h}" value="${option.value | h}"></td>
            <td>${option.name | h}</td>
        </tr>
        % endfor
    </table>
</div>
</%def>
