import datetime
from Candidate.Entity.Candidate import Candidate

from CandidateAlarm.Entity import CandidateAlarm
from User.Business.UserPermissionManager import UserPermissionManager
from lawrence.base import Handler
from lawrence.HTTPStatus import *

from User.Entity.UserPermission import UserPermission

class Index (Handler):
	__method__ = 'GET'
	__path__ = '/Candidate/Alarm/'

	def Handle(self):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'View')):
			raise Forbidden()

		return self.RenderTemplate('/CandidateAlarm/View/Index')

class Detail (Handler):
	__method__ = 'GET'
	__path__ = '/Candidate/Alarm/(?P<alarmId>[1-9][0-9]*)/'

	def Handle(self, *,
			alarm: CandidateAlarm):

		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'View')):
			raise Forbidden()

		if alarm.Candidate.status == 'enrolled' and not self.HasPermission(alarm.Candidate.path, 'admin'):
			self.CurrentUser.Grant(path = alarm.Candidate.path, permission = 'read')

		self.Model.CanEdit = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'Edit'))
		self.Model.CanDeleteNegotiation = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'DeleteNegotiation'))
		self.Model.CanEnroll = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'Enroll'))
		self.Model.CanCreateAppointment = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'CreateAppointment'))
		self.Model.CanDeleteAppointment = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'DeleteAppointment'))
		self.Model.CanPostComment = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'PostComment'))
		self.Model.CanPostRating = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'PostRating'))
		self.Model.CanImportGrades = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ImportGrades'))
		self.Model.CanServiceAppointment = self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'ServiceAppointment'))

		return self.RenderTemplate('/CandidateAlarm/View/Detail',
			alarm = alarm)

class Create (Handler):
	__method__ = 'POST'
	__path__ = '/Candidate/Alarm/Create'

	def Handle(self, *,
			candidate: Candidate,
			scheduledDate: datetime.date,
			content: str):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'Edit')):
			raise Forbidden()

		self.RequirePermission(candidate.Campus.Path, 'write')
		if candidate.status == 'enrolled':
			self.RequirePermission(candidate.Path, 'admin')

		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute('''
			INSERT INTO CandidateAlarm (CandidateId, UserId, ScheduledDate, Content)
			VALUES (%s, %s, %s, %s)''',
			[candidate.Id, self.CurrentUser.Id, scheduledDate, content])
		self.DatabaseCursor.Commit()

		raise NoContent()

class Delete (Handler):
	__method__ = 'POST'
	__path__ = '/Candidate/Alarm/(?P<alarmId>[1-9][0-9]*)/Delete'

	def Handle(self, *,
			alarm: CandidateAlarm):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'Edit')):
			raise Forbidden()

		self.RequirePermission(alarm.Candidate.Campus.Path, 'write')
		if alarm.Candidate.status == 'enrolled':
			self.RequirePermission(alarm.Candidate.Path, 'admin')

		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute('''
			DELETE FROM CandidateAlarm
			WHERE Id = %s''',
			[alarm.Id])
		self.DatabaseCursor.Commit()

		raise NoContent()

class Snooze (Handler):
	__method__ = 'POST'
	__path__ = '/Candidate/Alarm/(?P<alarmId>[1-9][0-9]*)/Snooze'

	def Handle(self, *,
			alarm: CandidateAlarm):
		if not self.GetManager(UserPermissionManager).HasPermission(
				self.CurrentUser,
				UserPermission(Group = 'Candidate', Permission = 'Edit')):
			raise Forbidden()

		self.RequirePermission(alarm.Candidate.Campus.Path, 'write')
		if alarm.Candidate.status == 'enrolled':
			self.RequirePermission(alarm.Candidate.Path, 'admin')

		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute('''
			UPDATE CandidateAlarm SET ScheduledDate = CURRENT_DATE + INTERVAL 1 DAY
			WHERE Id = %s''',
			[alarm.Id])

		self.DatabaseCursor.Commit()

		raise NoContent()