import datetime
from Campus.Business import CampusManager
from Campus.Entity import Campus

from lawrence.base import Handler
import lawrence.timeutil as timeutil
import lawrence.db as db

DBRow = db.DBRow
from Candidate.Entity.Candidate import Candidate
from Candidate.Entity.CandidateFilter import CandidateFilter

class AppointmentView (Handler):
	__method__ = 'GET'
	__path__ = '/report/appointment/'

	def Handle(self, *,
			date: datetime.date = None,
			user: int = None):
		self.RequirePermission('/report/appointment/', 'read')

		meetings = db.Execute('''
			SELECT UserId,
				   AppointmentDateTime,
				   Candidate.Id AS CandidateId,
				   Candidate.student_name AS student_name
			FROM CandidateAppointment
				 INNER JOIN Candidate
				   ON Candidate.Id = CandidateAppointment.CandidateId
			WHERE UserId IS NOT NULL''')

		enrollments = db.Execute('''
			SELECT CandidateAppointment.UserId,
				   Candidate.Id AS CandidateId,
				   Candidate.student_name AS student_name,
				   timestamp
			FROM CandidateAppointment
				 INNER JOIN Candidate
				   ON Candidate.Id = CandidateAppointment.CandidateId
				 INNER JOIN audit_candidate_status
				   ON Candidate.Id = audit_candidate_status.CandidateId
			WHERE status = 'enrolled' AND
				  NewValue = 'enrolled' AND
				  timestamp = (SELECT MAX(aux.timestamp)
							   FROM audit_candidate_status AS aux
							   WHERE aux.CandidateId = Candidate.Id AND
									 NewValue = 'enrolled') AND
				  CandidateAppointment.UserId IS NOT NULL AND
				  AppointmentDateTime = (SELECT MAX(aux.AppointmentDateTime)
										  FROM CandidateAppointment AS aux
										  WHERE aux.CandidateId = CandidateAppointment.CandidateId)''')

		if date is not None:
			firstDate = timeutil.PreviousMonth(date).replace(day = 21)
			lastDate = date.replace(day = 20)
			meetings = [meeting for meeting in meetings if firstDate <= meeting.AppointmentDateTime.date() <= lastDate]
			enrollments = [enrollment for enrollment in enrollments if firstDate <= enrollment.timestamp.date() <= lastDate]

		if user is not None:
			meetings = [meeting for meeting in meetings if meeting.UserId == user]
			enrollments = [enrollment for enrollment in enrollments if enrollment.UserId == user]

		interviewers = db.Execute('''
			SELECT Id,
				   FullName
			FROM User''')
		interviewers = {interviewer.Id: interviewer for interviewer in interviewers}

		for interviewer in list(interviewers.values()):
			interviewer.meetings = []
			interviewer.enrollments = []

		for meeting in meetings:
			interviewers[meeting.UserId].meetings.append(meeting)
		for enrollment in enrollments:
			interviewers[enrollment.UserId].enrollments.append(enrollment)

		for _, interviewer in list(interviewers.items()):
			if not interviewer.meetings and not interviewer.enrollments:
				del interviewers[interviewer.Id]

		if user is not None:
			return self.RenderTemplate('View/Report/Appointment/User',
					interviewer = interviewers[user])
		else:
			return self.RenderTemplate('View/Report/Appointment/Date',
					reportDate = date,
					interviewers = interviewers)

class StatusSnapshotView (Handler):
	__method__ = 'GET'
	__path__ = '/report/status/'

	def Handle(self):
		self.RequirePermission('/report/status/', 'read')

		statusCounts = {}
		campusList = self.DatabaseCursor.Execute('''
			SELECT Id, Name
			FROM Campus''')
		for campus in self.GetManager(CampusManager).GetAuthorized():
			candidateFilter = CandidateFilter(
				Page = None,
				Campus = [campus],
				Course = None,
				Segment = None,
				Status = None,
				AppointmentStatus = None,
				ExamDate = None,
				ExamTime = None,
				Tag = None,
				Query = None,
				SortOrder = None,
			)
			statusCounts[campus] = {}
			statusCounts[campus].update(Candidate.FilterCounts(self.CurrentUser,
				candidateFilter = candidateFilter)['Status'])
			candidateFilter.Status = 'present'
			statusCounts[campus].update(Candidate.FilterCounts(self.CurrentUser,
				candidateFilter = candidateFilter)['AppointmentStatus'])

		return self.RenderTemplate('View/Report/Status/Snapshot',
							 status_counts = statusCounts,
							 campusList = campusList)

class AppointmentCalendarView (Handler):
	__method__ = 'GET'
	__path__ = '/report/appointment/calendar/'

	def Handle(self, *,
			campus: Campus = None):
		raise NotImplementedError()
		# self.RequirePermission('/report/appointment/calendar/', 'read')
		#
		# campuses = db.Execute('''
		# 	SELECT Id,
		# 		   Name
		# 	FROM Campus
		# 	WHERE ''' + db.query_in('id', self.GetManager(CampusManager).GetAuthorized()) + '''
		# 	ORDER BY name''')
		#
		# if campus is None:
		# 	return self.RenderTemplate('Report/Appointment/Calendar/Index',
		# 						 campuses = campuses)
		#
		# appointments = db.Execute('''
		# 	SELECT DATE(AppointmentDateTime) AS appointment_date,
		# 		   TIME(AppointmentDateTime) AS appointment_time,
		# 		   AppointmentDateTime,
		# 		   Candidate.student_name,
		# 		   Candidate.Id AS CandidateId
		# 	FROM CandidateAppointment
		# 		 INNER JOIN Candidate
		# 		   ON Candidate.Id = CandidateAppointment.CandidateId
		# 	WHERE Candidate.CampusId = %s''',
		# 	(campus.id, ))
		#
		# return self.RenderTemplate('Report/Appointment/Calendar/Campus',
		# 					 campuses = campuses,
		# 					 appointments = appointments)

class AppointmentExamDateView (Handler):
	__method__ = 'GET'
	__path__ = '/report/appointment-exam-date/'

	def Handle(self):
		raise NotImplementedError()
		# self.RequirePermission('/report/', 'read')
		#
		# self.Model.AppointmentStatistics = {(row.CampusId, row.ExamDate): row for row in self.DatabaseCursor.Execute('''
		# 	SELECT
		# 		Candidate.CampusId,
		# 		DATE(Candidate.ExamDateTime) AS ExamDate,
		# 		COUNT(Candidate.Id) AS PresentCandidateCount,
		# 		COALESCE(SUM(CandidateAppointmentStatus.WasScheduled), 0) AS ScheduledCandidateCount,
		# 		COALESCE(SUM(CandidateAppointmentStatus.WasInterviewed), 0) AS InterviewedCandidateCount
		# 	FROM
		# 		Candidate
		# 		LEFT OUTER JOIN (SELECT
		# 							 CandidateId,
		# 							 CandidateId IS NOT NULL AS WasScheduled,
		# 							 MAX(UserId) IS NOT NULL AS WasInterviewed
		# 						 FROM CandidateAppointment
		# 						 GROUP BY CandidateId) AS CandidateAppointmentStatus
		# 		  ON CandidateAppointmentStatus.CandidateId = Candidate.Id
		# 	WHERE status = 'present' AND
		# 		  ExamDateTime IS NOT NULL
		# 	GROUP BY CampusId, ExamDate''')}
		#
		# self.Model.ExamDateList = [row.ExamDate for row in self.DatabaseCursor.Execute('''
		# 	SELECT DISTINCT
		# 		DATE(ExamDateTime) AS ExamDate
		# 	FROM
		# 		course_datetimes
		# 	WHERE
		# 		DATE(ExamDateTime) <= NOW()
		# 	ORDER BY ExamDate''')]
		#
		# self.Model.CampusList = self.DatabaseCursor.Execute('''
		# 	SELECT Id, Name
		# 	FROM Campus
		# 	ORDER BY Name''')  # FIXME: authorization
		#
		# return self.RenderTemplate('Report/Appointment/ExamDate')
