from lawrence.base import Handler
from lawrence.HTTPStatus import *
import lawrence.parse as parse
import lawrence.config as config
import lawrence.db as db

DBRow = db.DBRow

class Index (Handler):
    __method__ = 'GET'
    __path__ = '/settings/'

    def Handle(self):
        self.RequireLogin()

        return self.RenderTemplate('View/Settings/Index')

class MailView (Handler):
    __method__ = 'GET'
    __path__ = '/settings/mail/'

    def Handle(self):
        self.RequirePermission('/', 'admin')

        return self.RenderTemplate('View/Settings/Mail')

class MailEdit (Handler):
    __method__ = 'POST'
    __path__ = '/settings/mail/'

    def Handle(self, *,
			name: str,
			value: str):
        self.RequirePermission('/', 'admin')

        if name not in ('smtp_server', 'smtp_username', 'smtp_password'):
            raise BadRequest()

        setattr(config, name, value)

        raise NoContent()

class MailSMTPTest (Handler):
    __method__ = 'POST'
    __path__ = '/settings/mail/smtp-test/'

    def Handle(self):
        self.RequirePermission('/', 'admin')
        import smtplib
        try:
            smtpServer = smtplib.SMTP(config.smtp_server)
            smtpServer.starttls()
            smtpServer.login(config.smtp_username,
                              config.smtp_password)
            smtpServer.quit()
        except:
            raise InternalServerError()
        raise NoContent()

class EditSetting (Handler):
    __method__ = 'POST'
    __path__ = '/settings/'

    def Handle(self, *,
			name: str,
			value: str):
        self.RequirePermission('/', 'admin')

        if name not in ('FullName', 'short_name', 'contact_url',
                                  'contact_phone'):
            raise BadRequest()
        try:
            if name == 'contact_phone':
                value = parse.phone(value)
        except ValueError:
            raise BadRequest

        setattr(config, name, value)

        raise NoContent()

class ShiftsView (Handler):
    __method__ = 'GET'
    __path__ = '/settings/shifts/'

    def Handle(self, *,
			ajax: bool = False):
        self.RequirePermission('/', 'admin')

        shifts = self.DatabaseCursor.Execute('''
            SELECT Id, name AS Name
            FROM ClassShift
            ORDER BY Id''')
        if ajax:
            import json
            return json.dumps([{'value': shift.Id, 'text': shift.Name}
                               for shift in shifts])

        raise NotFound()
