import os.path
import datetime

from lawrence.base import Handler
import lawrence.db as db

DBRow = db.DBRow

EXPIRATION_TIME = 60 * 60

class StaticView (Handler):
    __method__ = 'GET'
    __path__ = '/static/.+'

    def Handle(self):
        if '?' in self.path:
            self.path = self.path.split('?')[0]
        try:
            with open(os.path.join(Handler._base_dir, self.path.lstrip('/')), 'rb') as staticFile:
                data = staticFile.read()
        except IOError:
            return
        if self.path.endswith('.css'):
            self.ContentType = 'text/css'
        elif self.path.endswith('.png'):
            self.ContentType = 'image/png'
        elif self.path.endswith('.jpg') or self.path.endswith('.jpeg'):
            self.ContentType = 'image/jpeg'
        elif self.path.endswith('.js'):
            self.ContentType = 'text/javascript'
        elif self.path.endswith('.woff'):
            self.ContentType = 'application/font-woff'
        elif self.path.endswith('.gif'):
            self.ContentType = 'image/gif'
        elif self.path.endswith('.xlsx'):
            self.ContentType = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
            self.Headers.append(('Content-Disposition', 'attachment'))
        self.Headers.append(('Cache-Control', 'max-age=%d, public' % EXPIRATION_TIME))
        self.Headers.append(('Expires',
                              (datetime.datetime.utcnow() +
                               datetime.timedelta(seconds = EXPIRATION_TIME))
                                  .strftime("%a, %d %b %Y %H:%M:%S GMT")))
        return data
