<%

# FIXME: move this somewhere more appropriate
import json

def date_js(date):
    timestamp = int(date.strftime('%s'))
    return (timestamp - timestamp % 86400) * 1000

def FillGaps(data):
    if len(data) == 0:
        return []
    minDate = min(date for date, count in data)
    maxDate = max(date for date, count in data)
    assert (maxDate - minDate) % (86400 * 1000) == 0
    missingDates = set(range(minDate, maxDate, 86400 * 1000)) - set(date for date, count in data)
    for missingDate in missingDates:
        data.append((missingDate, 0))
    return sorted(data)

currentData = FillGaps([(date_js(k), v) for k, v in sorted(Model.CurrentTimeSeries.items())])
if len(Model.CurrentTimeSeries) == 0 or len(Model.HistoricTimeSeries) == 0:
    dateOffset = 0
else:
    dateOffset = min(date_js(k) for k in Model.CurrentTimeSeries) - \
                 min(date_js(datum.RegistrationDate) for datum in Model.HistoricTimeSeries)
historicData = FillGaps([(date_js(datum.RegistrationDate) + dateOffset, datum.Count) for datum in Model.HistoricTimeSeries])

interestingDates = [date_js(datum.ExamDate) for datum in Model.ExamDateList]

%>

<div class="row-fluid">
    <div class="span12">
        <h2 class="box-header">Inscrições</h2>
        <div class="box">
            <div class="btn-group" data-toggle="buttons-radio">
                <a href="#" class="btn" onclick="renderPlot()">diárias</a>
                <a href="#" class="btn active" onclick="renderPlot('smooth')">semanais</a>
                <a href="#" class="btn" onclick="renderPlot('accumulate')">acumuladas</a>
            </div>
            <div id="registration-chart"></div>
        </div>
    </div>
</div>

<script>
var currentData = ${json.dumps(currentData)};
var historicData = ${json.dumps(historicData)};
var interestingDates = ${json.dumps(interestingDates)};
</script>

<script>
function smoothData(data) {
    var smoothedData = [];
    var history = [];
    var sum = 0;
    for (var i = 0; i < data.length; i++) {
        history.push(data[i][1]);
        sum += data[i][1];

        if (history.length > 7) {
            sum -= history[0];
            history = history.slice(1);
        }

        if (history.length == 7) {
            smoothedData.push([data[i][0], sum / 7]);
        }
    }

    return smoothedData;
}

function accumulateData(data) {
    var accumulatedData = [];
    var sum = 0;
    for (var i = 0; i < data.length; i++) {
        sum += data[i][1];
        accumulatedData.push([data[i][0], sum]);
    }

    return accumulatedData;
}

function renderPlot(mode) {
    var aggregate = function (data) { return data };
    if (mode == 'smooth') {
        aggregate = smoothData;
    } else if (mode == 'accumulate') {
        aggregate = accumulateData;
    }

    console.log(historicData);
    console.log(currentData);

    var chartSeries = [{
         label: 'Inscrições 2015',
         color: 'rgba(0, 136, 204, 0.3)',
         data: aggregate(historicData),
    }, {
        label: 'Inscrições 2016',
        color: 'rgba(0, 136, 204, 1.0)',
        data: aggregate(currentData),
    }];

    var latestDate = 0;
    for (var i = 0; i < historicData.length; i++) {
        latestDate = Math.max(latestDate, historicData[i][0]);
    }
    for (var i = 0; i < currentData.length; i++) {
        latestDate = Math.max(latestDate, currentData[i][0]);
    }

    var chartMarkings = []
    for (var i = 0; i < interestingDates.length; i++) {
        if (interestingDates[i] >= latestDate) {
            continue;
        }
        chartMarkings.push({
            color: "#ddd",
            lineWidth: 1,
            xaxis: {
                from: interestingDates[i],
                to: interestingDates[i]}});
    }

    var chart = $('#registration-chart');
    var wrapper = chart.closest('.graph-wrapper')[0]
    if (wrapper) {
        // FIXME
        wrapper.parentNode.replaceChild(chart[0], wrapper);
    }

    chart.simplePlot(chartSeries, {
        xaxis: {mode: 'time',
                minTickSize: [1, 'day'],
                timeformat: '%d/%b',
                show: false},
        yaxis: {minTickSize: 1,
                min: 0,
                tickDecimals: 0},
        grid: {markings: chartMarkings},
    }, {
        heightRatio: 0.25,
        legend: {show: false},
        tooltipText: (mode == 'smooth') ?
            "$.plot.formatDate(new Date(x), '%d/%b') + ': ' + y.toFixed(1) + ((y > 1) ? ' inscrições' : ' inscrição')" :
            "$.plot.formatDate(new Date(x), '%d/%b') + ': ' + y.toFixed(0) + ((y > 1) ? ' inscrições' : ' inscrição')",
    });

    var plot = $('#registration-chart')[0].plot;
    var o = null;
    var div = null;

    for (var i = 0; i < interestingDates.length; i++) {
        if (interestingDates[i] >= latestDate) {
            continue;
        }
        o = plot.pointOffset({x: interestingDates[i], y: 0});

        div = $('<div></div>')
                .css('left', o.left + 4)
                .css('top', o.top - 18)
                .css('position', 'absolute')
                .css('color', '#aaa')
                .css('font-size', 11);
        div.text($.plot.formatDate(new Date(interestingDates[i] + 86400000), '%d/%b'));
        chart.append(div);
    }
}

renderPlot('smooth');
</script>
