from Ozark import QueryAST

class DatabaseSchema:
	def __init__(self):
		self.Table = {}

def GetTableColumns(cursor, table):
	columnQuery = QueryAST.Query.Select(QueryAST.Table.SchemaTable(Name = 'columns', Schema = 'information_schema'))
	columnQuery.Column['Name'] = QueryAST.Expression.Column('column_name')
	columnQuery = columnQuery.Where(
		QueryAST.Expression.EqualTo(QueryAST.Expression.Column('table_schema'), table.Schema))
	columnQuery = columnQuery.Where(
		QueryAST.Expression.EqualTo(QueryAST.Expression.Column('table_name'), table.Name))
	for columnDescription in columnQuery.Execute(cursor):
		table.Column[columnDescription['Name']] = QueryAST.Expression.Column(**columnDescription)

def GetTableUniqueKeys(cursor, table):
	uniqueKeyQuery = QueryAST.Query.Select(QueryAST.Table.SchemaTable(Name = 'table_constraints', Schema = 'information_schema'))
	uniqueKeyQuery.Column['Name'] = QueryAST.Expression.Column('constraint_name')
	uniqueKeyQuery = uniqueKeyQuery.Where(
		QueryAST.Expression.EqualTo(QueryAST.Expression.Column('constraint_type'), 'UNIQUE'))
	uniqueKeyQuery = uniqueKeyQuery.Where(
		QueryAST.Expression.EqualTo(QueryAST.Expression.Column('table_schema'), table.Schema))
	uniqueKeyQuery = uniqueKeyQuery.Where(
		QueryAST.Expression.EqualTo(QueryAST.Expression.Column('table_name'), table.Name))
	for uniqueKeyDescription in uniqueKeyQuery.Execute(cursor):
		uniqueKey = QueryAST.TableConstraint.UniqueKey(**uniqueKeyDescription)
		uniqueKeyColumnQuery = QueryAST.Query.Select(QueryAST.Table.SchemaTable(Name = 'key_column_usage', Schema = 'information_schema'))
		uniqueKeyColumnQuery.Column['ColumnName'] = QueryAST.Expression.Column('column_name')
		uniqueKeyColumnQuery = uniqueKeyColumnQuery.Where(QueryAST.Expression.EqualTo(
			QueryAST.Expression.Column('constraint_name'), uniqueKey.Name))
		uniqueKeyColumnQuery = uniqueKeyColumnQuery.Where(QueryAST.Expression.EqualTo(
			QueryAST.Expression.Column('table_schema'), table.Schema))
		uniqueKeyColumnQuery = uniqueKeyColumnQuery.Where(QueryAST.Expression.EqualTo(
			QueryAST.Expression.Column('table_name'), table.Name))
		for row in uniqueKeyColumnQuery.Execute(cursor):
			uniqueKey.AddColumn(table.Column[row['ColumnName']])
		table.UniqueKeys.add(uniqueKey)

def GetSchema(cursor):
	schema = {}

	# Extract schema tables
	tableQuery = QueryAST.Query.Select()
	# tableInfo = DB.Table.SchemaTable(Name = 'tables', Schema = 'information_schema')
	# tableInfo.Column['table_schema'] = DB.Interface.Column(Name = 'table_schema')
	# tableInfo.Column['table_name'] = DB.Interface.Column(Name = 'table_name')
	# tableInfo = tableInfo.Rename(table_schema = 'Schema', table_name = 'Name')
	tableQuery.Column['Schema'] = QueryAST.Expression.Column('table_schema')
	tableQuery.Column['Name'] = QueryAST.Expression.Column('table_name')
	tableQuery = tableQuery.Join(QueryAST.Table.SchemaTable(Name = 'tables', Schema = 'information_schema'))
	tableQuery = tableQuery.Where(QueryAST.Expression.NotElementOf(QueryAST.Expression.Column('table_schema'), frozenset({'pg_catalog', 'information_schema'})))
	for tableDescription in tableQuery.Execute(cursor):
		table = QueryAST.Table.SchemaTable(**tableDescription)

		GetTableColumns(cursor, table)
		GetTableUniqueKeys(cursor, table)
		schema[table.Schema, table.Name] = table

	# TODO: rewrite this query as a relational query
	for table in schema.values():
		# TODO: this will fail for composite foreign keys
		# foreignKeyQuery = QueryAST.Query.Select()
		# foreignKeyQuery = foreignKeyQuery.Join(QueryAST.Table.SchemaTable('table_constraints', Schema = 'information_schema'))
		# foreignKeyQuery = foreignKeyQuery.Where(
		# 	QueryAST.Expression.EqualTo(QueryAST.Expression.Column('constraint_type'), 'UNIQUE'))
		# foreignKeyQuery = foreignKeyQuery.Where(
		# 	QueryAST.Expression.EqualTo(QueryAST.Expression.Column('table_schema'), table.Schema))
		# foreignKeyQuery = foreignKeyQuery.Where(
		# 	QueryAST.Expression.EqualTo(QueryAST.Expression.Column('table_name'), table.Name))
		###
		foreignKeyQuery = QueryAST.Query.Raw("""
			SELECT
				kcu.column_name AS "Column",
				ccu.table_schema AS "Schema",
				ccu.table_name AS "Name",
				ccu.column_name AS "ReferencedColumn"
			FROM
				information_schema.table_constraints
				JOIN information_schema.key_column_usage AS kcu USING (constraint_name)
				JOIN information_schema.constraint_column_usage AS ccu USING (constraint_name)
			WHERE
				information_schema.table_constraints.constraint_type = 'FOREIGN KEY' AND
				information_schema.table_constraints.table_schema = %(tableSchema)s AND
				information_schema.table_constraints.table_name = %(tableName)s""",
				tableSchema = table.Schema,
				tableName = table.Name)
		for row in foreignKeyQuery.Execute(cursor):
			# TODO: fetch the constraint name
			foreignKeyDescription = QueryAST.TableConstraint.ForeignKey(Name = None, Column = table.Column[row['Column']], ReferencedColumn = schema[row['Schema'], row['Name']].Column[row['Column']], ReferencedTable = schema[row['Schema'], row['Name']])
			table.ForeignKeys.add(foreignKeyDescription)
	_ = DatabaseSchema()
	_.Table = schema
	return schema