import http.cookies

from uuid import UUID
import json

class Response:
	def __init__(self, Body = (), *, Status: int = None, ContentType: str = None):
		if isinstance(Body, memoryview):
			self.Body = (bytes(Body), )
		elif isinstance(Body, (str, bytes,)):
			self.Body = (Body, )
		else:
			self.Body = Body
		if self.Body == ():
			self.Status = Status or 204
			self.ContentType = 'text/plain'
		else:
			self.Status = Status or 200
			self.ContentType = ContentType or 'text/html'
		self._Headers = []

	def _GetHeaders(self):
		contentType = self.ContentType
		if contentType.startswith('text/'):
			contentType += '; charset=utf-8'
		yield ('Content-Type', contentType)
		yield from self._Headers

	def AddHeader(self, headerName, headerValue):
		self._Headers.append((headerName, headerValue))

	def _AddCookieHeader(self, cookie):
		cookie['path'] = '/'
		cookie['secure'] = True
		cookie['httponly'] = True
		self.AddHeader('Set-Cookie', cookie.OutputString())

	def DeleteCookie(self, cookieName):
		cookie = http.cookies.Morsel()
		cookie.set(cookieName, '', '')
		cookie['max-age'] = 0
		self._AddCookieHeader(cookie)

	def AddCookie(self, cookieName, cookieValue):
		cookie = http.cookies.Morsel()
		cookie.set(cookieName, cookieValue, cookieValue)
		cookie['max-age'] = 2 * 60 * 60
		self._AddCookieHeader(cookie)

	def RedirectTo(self, destination):
		self.AddHeader('Location', destination)

	def SendHeaders(self, startResponse):
		startResponse({
			200: '200 OK',
			201: '201 Created',
			204: '204 No Data',
			303: '303 See Other',
			400: '400 Bad Request',
			403: '403 Forbidden',
			404: '404 Not Found',
			500: '500 Internal Server Error'}[self.Status], self._GetHeaders())

	def __iter__(self):
		yield from self.Body

class UUIDEncoder (json.JSONEncoder):
	def default(self, o):
		if isinstance(o, UUID):
			return str(o)
		return json.JSONEncoder.default(self, o)

class JSONResponse (Response):
	def __init__(self, Body, *, Status: int = None):
		Response.__init__(self, UUIDEncoder().encode(Body), Status = Status, ContentType = 'application/json')

class Redirect:
	def __init__(self, target):
		self.Target = target

	def ToResponse(self, *, routeRegistry):
		response = Response()
		target = routeRegistry.GetTarget(self.Target)
		response.AddHeader('Location', target)
		return response