from .Entity import Entity

class Anchor:
	"""Anchors are the archetypes for entities: they aggregate all possible
attributes and relationships that entities might have.

Anchors are *not* classes, but they can be called to create instances of
their related entities."""
	def __init__(self, *, name, table, key, parent = None, child = None):
		self._Table = table
		self._Key = key
		self._Entity = type(name, (Entity, ), {'_Anchor': self})
		self._Attribute = {}
		self._Tie = set()
		if parent is not None:
			self._Parent = dict(parent.items())
		else:
			self._Parent = dict()
		if child is not None:
			self._Child = dict(child.items())
		else:
			self._Child = dict()

	def __call__(self, **kwargs):
		return self._Entity(**kwargs)

	def __getitem__(self, key):
		return self._Attribute[key]

	def __setitem__(self, key, value):
		self._Attribute[key] = value

	def _AddEntityProperty(self, name, fget = None, fset = None, fdel = None, doc = None):
		"""Add a property to the entity associated to this anchor.

Except for the first argument (that describes the property name), the other
arguments are the same as those for :py:func:`property`."""
		setattr(self._Entity, name, property(fget, fset, fdel, doc))