from pymysql import IntegrityError
from Campus.Business import CampusManager
from SchoolClass.Business import SchoolClassManager
from SchoolClass.Entity import SchoolClass

from lawrence.base import Handler
from lawrence.HTTPStatus import *
import lawrence.parse as parse
import lawrence.db as db

DBRow = db.DBRow

class Index (Handler):
	__method__ = 'GET'
	__path__ = '/SchoolClass/'

	def Handle(self, *,
			json: str = None,
			CampusId: int = None):
		self.RequirePermission('/SchoolClass/', 'read')

		schoolClassList = self.DatabaseCursor.Execute('''
			SELECT
				SchoolClass.id,
				SchoolClass.name,
				Campus.id AS CampusId,
				Campus.name AS `Campus.Name`,
				ClassShift.id AS `shift.id`,
				ClassShift.name AS `shift.name`,
				SchoolClass.CourseId,
				(SchoolClassOutlay.MonthlyPayment IS NOT NULL) AS HasOutlay,
				(SchoolClassOutlay.MinimumDiscount = 0 AND
				 SchoolClassOutlay.AverageDiscount = 0 AND
				 SchoolClassOutlay.MaximumDiscount = 1) AS HasDefaultDiscounts,
				(SchoolClassOutlay.MinimumDiscount > SchoolClassOutlay.AverageDiscount OR
				 SchoolClassOutlay.AverageDiscount > SchoolClassOutlay.MaximumDiscount) AS IsDiscountPolicyCrossedOver
			FROM
				SchoolClass
				INNER JOIN Campus
				  ON SchoolClass.CampusId = Campus.id
				INNER JOIN Course
				  ON SchoolClass.CourseId = Course.id
				LEFT OUTER JOIN ClassShift
				  ON SchoolClass.ShiftId = ClassShift.id
				LEFT OUTER JOIN SchoolClassOutlay
				  ON SchoolClassOutlay.SchoolClassId = SchoolClass.id
			WHERE ''' + db.query_in('Campus.id', self.GetManager(CampusManager).GetAuthorized()) + '''
			ORDER BY
				`Campus.Name`,
				Course.SortOrder,
				name,
				ClassShift.id''')

		for schoolClass in schoolClassList:
			schoolClass.shift = DBRow(id = schoolClass['shift.id'], name = schoolClass['shift.name'])

		if CampusId is not None:
			schoolClassList = [schoolClass for schoolClass in schoolClassList if schoolClass.CampusId == CampusId]

		if json == 'select':
			import json
			return json.dumps([{'value': schoolClass.id, 'text': schoolClass.name + " " + schoolClass.shift.name}
							   for schoolClass in schoolClassList])

		authorizedCampusList = db.Execute('''
			SELECT
				Campus.id AS id,
				Campus.name AS name
			FROM
				Campus
			WHERE ''' + db.query_in('Campus.id', self.GetManager(CampusManager).GetAuthorized()))

		return self.RenderTemplate('View/SchoolClass/Index',
							 SchoolClass = schoolClassList,
							 Campus = authorizedCampusList)

class DetailView (Handler):
	__method__ = 'GET'
	__path__ = '/SchoolClass/(?P<schoolClassId>[1-9][0-9]*)/'

	def Handle(self, *,
			schoolClass: SchoolClass):
		self.RequirePermission(schoolClass.campus.path, 'read')

		if schoolClass.MinimumDiscount is not None and \
		   schoolClass.AverageDiscount is not None:
			self.Model.IsDiscountPolicyAvailable = True
			if schoolClass.MinimumDiscount > schoolClass.AverageDiscount:
				self.Model.AreDiscountsOutOfRange = True
			else:
				self.Model.AreDiscountsOutOfRange = False
				schoolClass.discounts = {}
				# FIXME
				for grade in range(schoolClass.MaximumScore + 1):
					schoolClass.discounts[grade] = \
							schoolClass.discount(grade)
				for grade in range(schoolClass.MaximumScore, 0, -1):
					if schoolClass.discounts[grade] == \
					   schoolClass.discounts[grade - 1]:
						del schoolClass.discounts[grade]
		else:
			self.Model.IsDiscountPolicyAvailable = False
			schoolClass.discounts = None

		self.Model.SchoolClass = schoolClass
		self.Model.IsDeletingAllowed = self.HasPermission(schoolClass.campus.path, 'admin')
		return self.RenderTemplate('View/SchoolClass/Detail',
							 date = getattr(self, 'date', None),
							 class_ = schoolClass)

class DetailEdit (Handler):
	__method__ = 'POST'
	__path__ = '/SchoolClass/(?P<schoolClassId>[1-9][0-9]*)/'

	def Handle(self, *,
			schoolClass: SchoolClass,
			name: str,
			value: str):
		self.RequirePermission('/', 'admin')

		if name in ('MonthlyPayment', 'MinimumDiscount', 'AverageDiscount', 'MaximumDiscount'):
			if name == 'MonthlyPayment':
				value = parse.money(value)
			else:
				value = parse.percentage(value)

			try:
				self.DatabaseCursor.Execute('''
					INSERT INTO SchoolClassOutlay (SchoolClassId)
					VALUES (%s)''',
					(schoolClass.id, ))
			except IntegrityError:
				pass

			self.DatabaseCursor.Execute(
				'''UPDATE SchoolClassOutlay
				   SET %s = %%s
				   WHERE SchoolClassId = %%s''' % (name, ),
				(value, schoolClass.id))
			self.DatabaseCursor.Commit()

			raise NoContent()

		if name not in ('name', 'campus', 'shift', 'course', 'Capacity'):
			raise BadRequest()
		if name in ('campus', 'shift', 'course'):
			name = name[0].upper() + name[1:]
			name += 'Id'
			value = value

		self.DatabaseCursor.Execute(
			'''UPDATE SchoolClass
			   SET %s = %%s
			   WHERE id = %%s''' % (name, ),
			(value, schoolClass.id))
		self.DatabaseCursor.Commit()

		raise NoContent()

class NewClass (Handler):
	__method__ = 'POST'
	__path__ = '/SchoolClass/Create'

	def Handle(self, *,
			Name: str,
			CampusId: int,
			CourseId: int):
		self.RequirePermission('/SchoolClass/', 'admin')

		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute(
			'''INSERT INTO SchoolClass (Name, CampusId, CourseId)
			   VALUES (%s, %s, %s)''',
			(Name,
			 CampusId,
			 CourseId))
		classId = self.DatabaseCursor.ScopeIdentity()
		self.DatabaseCursor.Commit()

		raise SeeOther('/SchoolClass/%s/' % classId)

class Delete (Handler):
	__method__ = 'POST'
	__path__ = '/SchoolClass/(?P<schoolClassId>[1-9][0-9]*)/Delete'

	def Handle(self, *,
			schoolClass: SchoolClass):
		self.RequirePermission(schoolClass.campus.path, 'admin')

		self.GetManager(SchoolClassManager).Delete(schoolClass)
		raise NoContent()

