from decimal import Decimal

import lawrence.config as config
from lawrence.Entity import Entity

class SchoolClass (Entity):
    def __init__(self, **kwargs):
        Entity.__init__(self, **kwargs)
        if 'id' in self:
            self.Id = self.id
            self.path = '/SchoolClass/%s/' % self.Id
        if 'campus' in self:  # FIXME
            self.campus.path = '/campus/%s/' % self.campus.id
        if 'course' in self:
            self.MaximumScore = 30

    def discount(self, score):
        def SchoolClassDiscount(score, maximumScore, y1, y2, x1 = Decimal('0.100'), x2 = Decimal('0.800')):
            """Return the discount for a student in a certain class.

            >>> from decimal import Decimal
            >>> SchoolClassDiscount(0, 35, Decimal('0.100'), Decimal('0.500'))
            Decimal('0.000')
            >>> SchoolClassDiscount(2, 35, Decimal('0.100'), Decimal('0.500'))
            Decimal('0.100')
            >>> SchoolClassDiscount(6, 35, Decimal('0.100'), Decimal('0.500'))
            Decimal('0.150')
            >>> SchoolClassDiscount(11, 35, Decimal('0.100'), Decimal('0.500'))
            Decimal('0.225')
            >>> SchoolClassDiscount(23, 35, Decimal('0.100'), Decimal('0.500'))
            Decimal('0.425')
            >>> SchoolClassDiscount(28, 35, Decimal('0.100'), Decimal('0.500'))
            Decimal('0.500')
            >>> SchoolClassDiscount(32, 35, Decimal('0.100'), Decimal('0.500'))
            Decimal('0.700')
            >>> SchoolClassDiscount(35, 35, Decimal('0.100'), Decimal('0.500'))
            Decimal('1.000')
            """
            assert 0 <= score <= maximumScore
            assert isinstance(y1, Decimal)
            assert isinstance(y2, Decimal)
            assert 0 <= x1 <= x2 <= 1
            assert 0 <= y1 <= y2 <= 1

            def RoundToStep(value, step = config.DiscountStep):
                """Round to the nearest multiple of step.

                >>> RoundToStep(Decimal('0.000'), Decimal('0.025'))
                Decimal('0.000')
                >>> RoundToStep(Decimal('0.100'), Decimal('0.025'))
                Decimal('0.100')
                >>> RoundToStep(Decimal('0.105'), Decimal('0.025'))
                Decimal('0.100')
                >>> RoundToStep(Decimal('0.115'), Decimal('0.025'))
                Decimal('0.125')
                >>> RoundToStep(Decimal('1.000'), Decimal('0.025'))
                Decimal('1.000')
                """
                assert isinstance(value, Decimal)
                assert isinstance(step, Decimal)
                assert 0 < step <= 1
                if value < 0:
                    return Decimal('0').quantize(step)
                elif 1 < value:
                    return Decimal('1').quantize(step)
                return ((value / step).to_integral() * step).quantize(step)

            if score == 0:
                return RoundToStep(Decimal(0))
            x = Decimal(score / float(maximumScore))
            if x <= x1:
                return y1
            if x1 < x <= x2:
                t = (x - x1) / (x2 - x1)
                return RoundToStep(y1 + (y2 - y1) * t)
            t = ((x - x2) / (1 - x2))
            return RoundToStep(y2 + t * t * (1 - y2) + (y2 - y1) / (x2 - x1) * (x - x2) * (1 - t))
        return SchoolClassDiscount(score, self.MaximumScore, self.MinimumDiscount, self.AverageDiscount)

