#!/usr/bin/env python2.7
# encoding: utf-8

import lawrence.db as db
from lawrence.util import DecodeId

from .EntityFromXlsx import Entity, ReadWorksheet
from MySQLdb import OperationalError, IntegrityError

def NormalizeForMatch(s):
    return s.lower()  # FIXME

def MatchesName(s, t):
    return s == t
    return (" " + NormalizeForMatch(s) + " ") in (" " + NormalizeForMatch(t) + " ")

def BestMatch(needle, haystack, key = lambda entity: entity.Name):
    matchingEntityList = [entity for entity in haystack if MatchesName(needle, key(entity))]
    if len(matchingEntityList) == 1:
        return matchingEntityList[0]
    elif len(matchingEntityList) == 0:
        raise IndexError('match not found')
    else:
        raise IndexError('more than one match found')

cursor = db.Cursor()
farabeCandidateList = cursor.Execute('''
    SELECT
        id AS Id,
        student_name AS Name
    FROM Candidate
    WHERE status != 'cancelled' ''')

farabeTagList = cursor.Execute('''
    SELECT
        id AS Id,
        name AS Name
    FROM CandidateTag''')

def ImportTags(workbook):
    global farabeTagList

    for entity in ReadWorksheet(workbook.get_sheet_by_name(name = "Plan1"), {
        'InscricaoFarabe': str,
        'NomeAluno': str,
        'Marcador': str,
    }):
        try:
            matchingTag = BestMatch(entity.Marcador, farabeTagList)
        except IndexError as ex:
            if str(ex) != 'match not found':
                raise
            cursor = db.Cursor()
            cursor.Begin()
            cursor.Execute('''
                INSERT INTO CandidateTag (Name)
                VALUES (%s)''',
                (entity.Marcador, ))
            farabeTagList = cursor.Execute('''
                SELECT
                    id AS Id,
                    name AS Name
                FROM CandidateTag''')
            cursor.SaveChanges()

def ImportSingleStudent(i, entity):
    while True:
        try:
            if entity.InscricaoFarabe and len(entity.InscricaoFarabe) == 8:
                matchingCandidate = Entity(Id = DecodeId(int(entity.InscricaoFarabe)))
            else:
                try:
                    matchingCandidate = BestMatch(entity.NomeAluno, farabeCandidateList)
                except IndexError as ex:
                    print(i+2, entity, ex)
                    return

            matchingTag = BestMatch(entity.Marcador, farabeTagList)

            cursor = db.Cursor()
            cursor.Begin()

            try:
                cursor.Execute('''
                    INSERT INTO CandidateTagMap (CandidateId, TagId)
                    VALUES (%s, %s)''',
                    (matchingCandidate.Id, matchingTag.Id))
            except IntegrityError:
                pass

            cursor.SaveChanges()
            return
        except OperationalError:
            pass

def ImportStudents(workbook):
    for i, entity in enumerate(ReadWorksheet(workbook.get_sheet_by_name(name = "Plan1"), {
        'InscricaoFarabe': str,
        'NomeAluno': str,
        'Marcador': str,
    })):
        ImportSingleStudent(i, entity)

def _main():
    import sys
    from openpyxl.reader.excel import load_workbook
    with open(sys.argv[1], 'rb') as inputFile:
        workbook = load_workbook(inputFile)

    ImportTags(workbook)
    ImportStudents(workbook)

if __name__ == '__main__':
    _main()
