#!/usr/bin/env python2.7
# encoding: utf-8

from collections import namedtuple
import lawrence.db as db

from .EntityFromXlsx import ReadWorksheet
from MySQLdb import OperationalError
DBRow = db.DBRow

def NormalizeForMatch(s):
    return s.lower()  # FIXME

def MatchesName(s, t):
    return (" " + NormalizeForMatch(s) + " ") in (" " + NormalizeForMatch(t) + " ")

def BestMatch(needle, haystack, key = lambda entity: entity.Name):
    matchingEntityList = [entity for entity in haystack if MatchesName(needle, key(entity))]
    matchingEntityList.sort(key = lambda entity: len(key(entity)))
    if len(matchingEntityList) >= 1:
        return matchingEntityList[0]
    else:
        raise IndexError('match not found')

CampaignShift = namedtuple('CampaignShift', ('Campaign', 'Shift'))

def ImportSingleStudent(entity):
    while True:
        try:
            cursor = db.Cursor()
            cursor.Begin()

            matchingCampus = DBRow(Id = 4)
            matchingCampaign = DBRow(Id = 325)

            cursor.Execute('''
                INSERT INTO Candidate (status, contact_name, contact_email, student_name, school, ExamCampusId,
                            next_call_time, guardian_cpf, guardian_present, got_result, RegistrationDateTime)
                VALUES ('present', %s, %s, %s, NULL, %s, NOW(), NULL, 0, 0, NOW())''',
                (entity.Nome, entity.EMail, entity.Nome, matchingCampus.Id))
            candidateId = cursor.ScopeIdentity()

            cursor.Execute('''
                UPDATE Candidate
                SET
                    CampaignId = %s,
                    CampusId = %s
                WHERE Id = %s''',
                (matchingCampaign.Id, matchingCampus.Id, candidateId))
            for phone in entity.Telefone:
                cursor.Execute('''
                    INSERT INTO CandidatePhone (CandidateId, phone)
                    VALUES (%s, %s)''',
                    (candidateId, '0' + phone))

            if entity.CloseCall:
                cursor.Execute('''
                    INSERT INTO CandidateTagMap (CandidateId, TagId)
                    VALUES (%s, %s)''',
                    (candidateId, 22))
            else:
                cursor.Execute('''
                    INSERT INTO CandidateTagMap (CandidateId, TagId)
                    VALUES (%s, %s)''',
                    (candidateId, 23))

            cursor.SaveChanges()
            return candidateId
        except OperationalError:
            pass

def ImportStudents(workbook):
    count = 0
    for entity in ReadWorksheet(workbook.get_sheet_by_name(name = "Base de Contatos"), {
        'Nome': str,
        'EMail': str,
        'Telefone': lambda x: str(x).split(', '),
        'CloseCall': bool,
    }):
        count += 1
        print(count)
        print(entity)
        print(ImportSingleStudent(entity))

def _main():
    import sys
    from openpyxl.reader.excel import load_workbook
    with open(sys.argv[1], 'rb') as inputFile:
        workbook = load_workbook(inputFile)

    ImportStudents(workbook)

if __name__ == '__main__':
    _main()
