#!/usr/bin/env python2.7
# encoding: utf-8

"""Generate all logo versions that are used in page templates."""

import sys
import PIL
import PIL.Image
import PIL.ImageChops

def scale_to_height(source, height):
    assert height > 0
    assert isinstance(source, PIL.Image.Image)
    aspect_ratio = source.size[0] / float(source.size[1])
    output_size = (int(round(aspect_ratio * height)), height)
    return source.resize(output_size, PIL.Image.ANTIALIAS)

def brand_logo(source):
    assert source.mode == 'L'
    mask = scale_to_height(source, 30)

    shadow = PIL.Image.new('RGBA', mask.size, (0, 0, 0, 0))
    shadow.paste((0, 0, 0, 51), (0, -1), mask)

    foreground = PIL.Image.new('RGBA', mask.size, (255, 255, 255, 0))
    foreground.paste((255, 255, 255, 255), (0, 0), mask)

    composite = PIL.Image.new('RGBA', mask.size, (0, 0, 0, 0))
    composite = PIL.Image.alpha_composite(composite, shadow)
    composite = PIL.Image.alpha_composite(composite, foreground)

    return composite

def signin_logo(source):
    assert source.mode == 'L'
    mask = scale_to_height(source, 55)

    shadow = PIL.Image.new('RGBA', mask.size, (0, 0, 0, 0))
    shadow.paste((0, 0, 0, 204), (0, -1), mask)

    foreground = PIL.Image.new('RGBA', mask.size, (255, 255, 255, 0))
    foreground.paste((255, 255, 255, 255), (0, 0), mask)

    composite = PIL.Image.new('RGBA', mask.size, (0, 0, 0, 0))
    composite = PIL.Image.alpha_composite(composite, shadow)
    composite = PIL.Image.alpha_composite(composite, foreground)

    return composite

def error_logo(source):
    assert source.mode == 'L'
    mask = scale_to_height(source, 30)

    shadow = PIL.Image.new('RGBA', mask.size, (255, 255, 255, 0))
    shadow.paste((255, 255, 255, 51), (0, -1), mask)

    foreground = PIL.Image.new('RGBA', mask.size, (0, 0, 0, 0))
    foreground.paste((0, 0, 0, 255), (0, 0), mask)

    composite = PIL.Image.new('RGBA', mask.size, (0, 0, 0, 0))
    composite = PIL.Image.alpha_composite(composite, shadow)
    composite = PIL.Image.alpha_composite(composite, foreground)

    return composite

def _main():
    image = PIL.Image.open(sys.argv[1]).convert('RGBA').split()[3]
    brand_logo(image).save('static/logo.png')
    signin_logo(image).save('static/logo-signin.png')  # FIXME: remove text
    error_logo(image).save('static/logo-error.png')

if __name__ == '__main__':
    _main()

