from lawrence.base import EntityManager

from User.Entity.User import User
from User.Entity.UserRole import UserRole
from User.Entity.UserPermission import UserPermission

class UserPermissionManager (EntityManager):
	__entity__ = UserPermission

	def GetAll(self):
		return [UserPermission(**row) for row in self.DatabaseCursor.Execute('''
			SELECT DISTINCT
				Permission.Id,
				PermissionGroup.Identifier AS `Group`,
				Permission.Identifier,
				Permission.Name
			FROM
				Permission
				INNER JOIN PermissionGroup
				  ON Permission.PermissionGroupId = PermissionGroup.Id''')]

	def GetById(self, permissionId):
		assert isinstance(permissionId, int)

		return UserPermission(**self.DatabaseCursor.Single('''
			SELECT DISTINCT
				Permission.Id,
				PermissionGroup.Identifier AS `Group`,
				Permission.Identifier,
				Permission.Name
			FROM
				Permission
				INNER JOIN PermissionGroup
				  ON Permission.PermissionGroupId = PermissionGroup.Id
			WHERE Permission.Id = %s''',
			(permissionId, )))

	def GetUserPermissions(self, user):
		assert isinstance(user, User)

		return [UserPermission(**row) for row in self.DatabaseCursor.Execute('''
			SELECT DISTINCT
				Permission.Id,
				PermissionGroup.Identifier AS `Group`,
				Permission.Identifier AS Permission,
				Permission.Name AS Name
			FROM
				Permission
				INNER JOIN PermissionGroup
				  ON Permission.PermissionGroupId = PermissionGroup.Id
				INNER JOIN RolePermissionMap
				  ON RolePermissionMap.PermissionId = Permission.Id
				INNER JOIN Role
				  ON RolePermissionMap.RoleId = Role.Id
				INNER JOIN UserRoleMap
				  ON UserRoleMap.RoleId = Role.Id
				INNER JOIN User
				  ON UserRoleMap.UserId = User.Id
			WHERE
				User.Id = %s''',
			(user.Id, ))]

	def GetRolePermissions(self, role):
		assert isinstance(role, UserRole)

		return [UserPermission(**row) for row in self.DatabaseCursor.Execute('''
			SELECT DISTINCT
				Permission.Id,
				PermissionGroup.Identifier AS `Group`,
				Permission.Identifier AS Permission,
				Permission.Name AS Name
			FROM
				Permission
				INNER JOIN PermissionGroup
				  ON Permission.PermissionGroupId = PermissionGroup.Id
				INNER JOIN RolePermissionMap
				  ON RolePermissionMap.PermissionId = Permission.Id
				INNER JOIN Role
				  ON RolePermissionMap.RoleId = Role.Id
			WHERE
				Role.Id = %s''',
			(role.Id, ))]

	def HasPermission(self, user, permission):
		if user is None:
			return False
		assert isinstance(user, User)
		assert isinstance(permission, UserPermission)

		matchingPermissions = self.DatabaseCursor.Execute('''
			SELECT DISTINCT
				Permission.Id,
				PermissionGroup.Identifier AS `Group`,
				Permission.Identifier AS Permission,
				Permission.Name AS Name
			FROM
				Permission
				INNER JOIN PermissionGroup
				  ON Permission.PermissionGroupId = PermissionGroup.Id
				INNER JOIN RolePermissionMap
				  ON RolePermissionMap.PermissionId = Permission.Id
				INNER JOIN Role
				  ON RolePermissionMap.RoleId = Role.Id
				INNER JOIN UserRoleMap
				  ON UserRoleMap.RoleId = Role.Id
				INNER JOIN User
				  ON UserRoleMap.UserId = User.Id
			WHERE
				PermissionGroup.Identifier = %s AND
				Permission.Identifier = %s AND
				User.Id = %s''',
			(permission.Group, permission.Permission, user.Id))

		return len(matchingPermissions) > 0
