from lawrence.base import EntityManager
from User.Entity.UserRole import UserRole

from User.Entity.User import User
from User.Entity.UserPermission import UserPermission

class UserRoleManager (EntityManager):
	__entity__ = UserRole

	def GetAll(self):
		return [UserRole(**row) for row in self.DatabaseCursor.Execute('''
			SELECT DISTINCT
				Role.Id AS `Id`,
				Role.Name AS `Name`
			FROM
				Role''')]

	def GetById(self, roleId):
		assert isinstance(roleId, int)

		return UserRole(**self.DatabaseCursor.Single('''
			SELECT DISTINCT
				Role.Id AS `Id`,
				Role.Name AS `Name`
			FROM
				Role
			WHERE Role.Id = %s''',
			(roleId, )))

	def Add(self, role):
		assert isinstance(role, UserRole)

		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute('''
			INSERT INTO Role (Name)
			VALUES (%s)''',
			(role.Name, ))
		role.Id = self.DatabaseCursor.ScopeIdentity()
		self.DatabaseCursor.Commit()

	def SetPermission(self, role, permission, grant):
		assert isinstance(role, UserRole)
		assert isinstance(permission, UserPermission)
		assert isinstance(grant, bool)

		self.DatabaseCursor.Begin()
		self.DatabaseCursor.Execute('''
			DELETE FROM RolePermissionMap
			WHERE
				RolePermissionMap.RoleId = %s AND
				RolePermissionMap.PermissionId = %s''',
			(role.Id, permission.Id))
		if grant:
			self.DatabaseCursor.Execute('''
				INSERT INTO RolePermissionMap (RoleId, PermissionId)
				VALUES (%s, %s)''',
				(role.Id, permission.Id))
		self.DatabaseCursor.Commit()

	def AddPermission(self, role, permission):
		assert isinstance(role, UserRole)
		assert isinstance(permission, UserPermission)

		self.SetPermission(role, permission, True)

	def RemovePermission(self, role, permission):
		assert isinstance(role, UserRole)
		assert isinstance(permission, UserPermission)

		self.SetPermission(role, permission, False)

	def GetUserRoles(self, user):
		assert isinstance(user, User)

		return [UserRole(**row) for row in self.DatabaseCursor.Execute('''
			SELECT DISTINCT
				Role.Id AS `Id`,
				Role.Name AS `Name`
			FROM
				Role
				INNER JOIN UserRoleMap
				  ON UserRoleMap.RoleId = Role.Id
				INNER JOIN User
				  ON UserRoleMap.UserId = User.Id
			WHERE
				User.Id = %s''',
			(user.Id, ))]
