#!/usr/bin/env python2.7
# encoding: utf-8

from lawrence.Monte.WordUtil import *
from lawrence.Monte.Chunk import Question
from lawrence.Monte.ImportedDocument import ImportedDocument

class Monte (object):
    def __init__(self, template, examTitle):
        self.Template = ZipFile(template, 'r')
        self.ExamTitle = examTitle

        for name, xmlFile in ZipContents(self.Template):
            if name == 'word/document.xml':
                self.TemplateTree = ElementTree.parse(xmlFile)
                templateBody = self.TemplateTree.find(ns('w:body'))
                for el in templateBody[:]:
                    RemoveRevisions(el)
                    if IsEmpty(el):
                        templateBody.remove(el)
            elif name == 'word/numbering.xml':
                self.NumberingTree = ElementTree.parse(xmlFile)
                usedIdList = [0]
                for el in self.NumberingTree.findall(ns('.//w:abstractNum')):
                    usedIdList.append(int(GetProp(el, '[w:abstractNumId]')))
                self.NextAbstractNumId = max(usedIdList) + 1
                usedIdList = [0]
                for el in self.NumberingTree.findall(ns('.//w:num')):
                    usedIdList.append(int(GetProp(el, '[w:numId]')))
                self.NextNumId = max(usedIdList) + 1
            elif name == 'word/styles.xml':
                self.StylesTree = ElementTree.parse(xmlFile)
            elif name == 'word/_rels/document.xml.rels':
                self.RelationshipTree = ElementTree.parse(xmlFile)
            elif name == '[Content_Types].xml':
                self.ContentTypesTree = ElementTree.parse(xmlFile)
        self.FindStyles()
        self.SupplementContentTypes()

        self.Chunks = []
        self.Questions = 0
        self.ImportedMedia = {}

    def ImportDocument(self, source, filename = None):
        ImportedDocument(self, source, filename)

    def SupplementContentTypes(self):
        _EXTENSIONS = {
            'png': 'image/png',
            'jpeg': 'image/jpeg',
            'jpg': 'image/jpeg',
            'gif': 'image/gif',
            'wmf': 'image/x-wmf',
            'bin': 'application/vnd.openxmlformats-officedocument.oleObject',
        }
        knownExtensions = set()
        for el in self.ContentTypesTree.findall(ns('.//ct:Default')):
            knownExtensions.add(GetProp(el, 'Extension'))
        for ext in set(_EXTENSIONS) - knownExtensions:
            default = Element('ct:Default')
            SetProp(default, 'ContentType', _EXTENSIONS[ext])
            SetProp(default, 'Extension', ext)
            self.ContentTypesTree.getroot().insert(0, default)

    def FindStyles(self):
        self.ChoiceStyle = None
        self.QuestionStyle = None
        self.SubjectStyle = None
        self.ExamTitleStyle = None
        self.ExamModelStyle = None
        self.CitationStyle = None
        self.AnswerStyle = None
        for style in self.StylesTree.findall(ns('.//w:style')):
            aliases = [GetProp(style, 'w:name[w:val]')]
            for alias in style.findall(ns('w:aliases')):
                aliases.append(GetProp(alias, '[w:val]'))
            if "Alternativa" in aliases:
                self.ChoiceStyle = style
            if "Questão" in aliases:
                self.QuestionStyle = style
            if "Matéria" in aliases:
                self.SubjectStyle = style
            if "Título da prova" in aliases:
                self.ExamTitleStyle = style
            if "Modelo da prova" in aliases:
                self.ExamModelStyle = style
            if "Fonte" in aliases:
                self.CitationStyle = style
            if "Resposta" in aliases:
                self.AnswerStyle = style

    def NumberingStyle(self, el):
        if not el:
            return None
        numPr = el.find(ns('.//w:numPr'))
        if numPr is None:
            return None
        numId = GetProp(numPr, 'w:numId[w:val]')
        ilvl = GetProp(numPr, 'w:ilvl[w:val]')
        num = SearchProp(self.NumberingTree, 'w:num[w:numId]', numId)
        if num is None:
            return None
        abstractNumId = GetProp(num, 'w:abstractNumId[w:val]')
        abstractNum = SearchProp(self.NumberingTree,
                                   'w:abstractNum[w:abstractNumId]',
                                   abstractNumId)
        lvl = SearchProp(abstractNum, 'w:lvl[w:ilvl]', ilvl)
        if lvl is None:
            return None
        return GetProp(lvl, 'w:numFmt[w:val]')

    def ImportMedia(self, name, data):
        mediaPrefix = 'word/imported/%d/' % (len(self.ImportedMedia), )
        self.ImportedMedia[mediaPrefix + name] = data
        return mediaPrefix[5:] + name

    def AddRelationship(self, relationshipType, target):
        usedIds = [0]
        for el in self.RelationshipTree.findall(ns('.//rel:Relationship')):
            usedIds.append(int(GetProp(el, '[Id]')[3:]))
        freeId = 'rId' + str(max(usedIds) + 1)

        relationship = Element('rel:Relationship')
        SetProp(relationship, 'Id', freeId)
        SetProp(relationship, 'Type', relationshipType)
        SetProp(relationship, 'Target', target)
        self.RelationshipTree.getroot().append(relationship)
        return freeId

    def AllocateChoiceNumId(self):
        numId = GetProp(self.ChoiceStyle,
                          'w:pPr/w:numPr/w:numId[w:val]')
        num = SearchProp(self.NumberingTree, 'w:num[w:numId]', numId)
        abstractNumId = GetProp(num, 'w:abstractNumId[w:val]')
        abstractNum = SearchProp(self.NumberingTree,
                                   'w:abstractNum[w:abstractNumId]',
                                   abstractNumId)
        abstractNumId = self.AddAbstractNum(abstractNum)
        return self.AddNum(abstractNumId)

    def AddAbstractNum(self, abstractNum):
        abstractNum = copy.deepcopy(abstractNum)
        freeId = str(self.NextAbstractNumId)
        self.NextAbstractNumId += 1
        SetProp(abstractNum, '[w:abstractNumId]', freeId)
        self.NumberingTree.getroot().insert(0, abstractNum)
        return freeId

    def AddNum(self, abstractNumId):
        freeId = str(self.NextNumId)
        self.NextNumId += 1
        num = Element('w:num')
        SetProp(num, '[w:numId]', freeId)
        SetProp(num, 'w:abstractNumId[w:val]', abstractNumId)
        for levelId in range(9):
            lvlOverride = Element('w:lvlOverride')
            SetProp(lvlOverride, '[w:ilvl]', str(levelId))
            SetProp(lvlOverride, 'w:startOverride[w:val]', '1')
            num.append(lvlOverride)
        self.NumberingTree.getroot().append(num)
        return str(freeId)

    def WriteModel(self, modelName, examFile, answerFile):
        def UpdateHeader(headerFile):
            headerTree = ElementTree.parse(headerFile)
            for paragraph in headerTree.getroot().findall(ns('.//w:p')):
                if GetProp(paragraph, 'w:pPr/w:pStyle[w:val]') == GetProp(self.ExamModelStyle, '[w:styleId]'):
                    paragraph[:] = Paragraph(modelName, style = self.ExamModelStyle)[:]
                if GetProp(paragraph, 'w:pPr/w:pStyle[w:val]') == GetProp(self.ExamTitleStyle, '[w:styleId]'):
                    paragraph[:] = Paragraph(self.ExamTitle, style = self.ExamTitleStyle)[:]
            return DumpTree(headerTree)

        with ZipFile(examFile, 'w') as examDocx:
            for name, xmlFile in ZipContents(self.Template):
                if name == 'word/document.xml':
                    examTree = copy.deepcopy(self.TemplateTree)
                    examBody = examTree.find(ns('w:body'))
                    for chunk in self.Chunks:
                        for el in chunk.ExamContent():
                            examBody.append(el)
                    data = DumpTree(examTree)
                elif name.startswith(('word/header', 'word/footer')):
                    data = UpdateHeader(xmlFile)
                elif name == 'word/numbering.xml':
                    data = DumpTree(self.NumberingTree)
                elif name == '[Content_Types].xml':
                    data = DumpTree(self.ContentTypesTree)
                elif name == 'word/_rels/document.xml.rels':
                    data = DumpTree(self.RelationshipTree)
                else:
                    data = xmlFile.read()
                examDocx.writestr(name, data)
            for name in self.ImportedMedia:
                examDocx.writestr(name, self.ImportedMedia[name])

        with ZipFile(answerFile, 'w') as answerDocx:
            for name, xmlFile in ZipContents(self.Template):
                if name == 'word/document.xml':
                    answerTree = copy.deepcopy(self.TemplateTree)
                    answerBody = answerTree.find(ns('w:body'))
                    for chunk in self.Chunks:
                        for el in chunk.AnswerContent():
                            answerBody.append(el)
                    answerDocx.writestr(name, DumpTree(answerTree))
                elif name.startswith(('word/header', 'word/footer')):
                    answerDocx.writestr(name, UpdateHeader(xmlFile))
                else:
                    answerDocx.writestr(name, xmlFile.read())

        # Shuffle questions for next model.
        for chunk in self.Chunks:
            if isinstance(chunk, Question):
                chunk.Shuffle()

    def WritePackage(self, modelNames, packageFile):
        with ZipFile(packageFile, 'w') as packageZip:
            for modelName in modelNames:
                examBuffer, answersBuffer = StringIO(), StringIO()
                self.WriteModel(modelName, examBuffer, answersBuffer)
                packageZip.writestr('prova_%s.docx' % modelName, examBuffer.getvalue())
                packageZip.writestr('gabarito_%s.docx' % modelName, answersBuffer.getvalue())
