import copy
import os
import pickle
import re
import sys
import uuid
import datetime

from mako.lookup import TemplateLookup
from paste.request import get_cookies, parse_formvars
from paste.util.multidict import MultiDict
from Ozark.Injector import InjectorFactory

import lawrence.config as config
import lawrence.db as db
from pymysql import OperationalError
import lawrence.parse as parse
from lawrence.HTTPStatus import *
from lawrence.Entity import Entity
DBRow = db.DBRow

# def FieldParser(fieldType):
# 	if fieldType is IOBase:
# 		return parse.file_
# 	return fieldType

def NumberFactory(*, argumentName: str, argumentType: type, formVars):
	fieldValues = [v for k, v in formVars if k == argumentName]
	if argumentType is bool:
		return list(map(lambda value: {'true': True, 'false': False, '0': False, '1': True}[value.lower()], fieldValues))
	elif argumentType is int:
		return list(map(int, fieldValues))
	else:
		return list(map(float, fieldValues))

def StringFactory(*, argumentName: str, argumentType: type, formVars):
	fieldValues = [v for k, v in formVars if k == argumentName]
	return list(map(str, fieldValues))

def EntityFactory(*,
		argumentName: str,
		argumentType: type,
		currentUser,
		databaseCursor,
		formVars):
	fieldValues = [int(v) for k, v in formVars if k == argumentName + 'Id']
	return [application.Manager[argumentType](currentUser = currentUser, databaseCursor = databaseCursor).GetById(entityId) for entityId in fieldValues]

def DateFactory(*, argumentName: str, formVars):
	fieldValues = [v for k, v in formVars if k == argumentName]
	return list(map(parse.date, fieldValues))

def TimeFactory(*, argumentName: str, formVars):
	fieldValues = [v for k, v in formVars if k == argumentName]
	return list(map(parse.time, fieldValues))

def DateTimeFactory(*, argumentName: str, formVars):
	fieldValues = [v for k, v in formVars if k == argumentName]
	return list(map(parse.datetime, fieldValues))

def BytesFactory(*, argumentName: str, formVars):
	fieldValues = [v.file.read() for k, v in formVars if k == argumentName]
	return list(fieldValues)

class Application (object):
	def __init__(self):
		self.InjectorFactory = InjectorFactory()
		self.InjectorFactory.AddByTypeArgumentFactory(NumberFactory, int)
		self.InjectorFactory.AddByTypeArgumentFactory(BytesFactory, bytes)
		self.InjectorFactory.AddByTypeArgumentFactory(StringFactory, str)
		self.InjectorFactory.AddByTypeArgumentFactory(EntityFactory, Entity)
		self.InjectorFactory.AddByTypeArgumentFactory(DateTimeFactory, datetime.datetime)
		self.InjectorFactory.AddByTypeArgumentFactory(DateFactory, datetime.date)
		self.InjectorFactory.AddByTypeArgumentFactory(TimeFactory, datetime.time)

		self.Manager = {}
		self._root = None
		self.TemplateLookup = None
		self.Controllers = {'GET': [], 'POST': [], 'OPTIONS': []}

	@property
	def Root(self):
		return self._root

	@Root.setter
	def Root(self, value):
		self._root = value
		Handler._base_dir = value
		self.TemplateLookup = TemplateLookup(
			directories = [self._root],
			input_encoding = 'utf-8',
			output_encoding = 'utf-8',
			module_directory = os.path.join(self._root, 'cache/View/'),
			imports = ['from lawrence.util import js',
					   'from lawrence.util import gender',
					   'from lawrence.util import decimal',
					   'from lawrence.util import percentage',
					   'from lawrence.util import int_',
					   'from lawrence.util import br',
					   'from lawrence.util import QueryString',
					   'from lawrence.util import money',
					   'from lawrence.util import cpf',
					   'from lawrence.util import phone',
					   'from lawrence.util import TimeDelta',
					   'from lawrence.util import time',
					   'from lawrence.util import ShortTime',
					   'from lawrence.util import unit',
					   'from lawrence.util import ShortDay, LongDay',
					   'from lawrence.util import ShortMonth, LongMonth',
					   'from lawrence.util import year',
					   'from lawrence.util import date_s, LongDateTime',
					   'from lawrence.util import group_by',
					   'from lawrence.util import n_partition',
					   'from lawrence.util import columns, selected',
					   'from lawrence.util import checked',
					   'from lawrence.util import disabled',
					   'from lawrence.util import AgeInYearsAndMonths',
					   'from decimal import Decimal',
					   'import lawrence.timeutil as timeutil',
					   'import lawrence.db as db',
					   'from lawrence.db import DBRow',
					  ])

	def __call__(self, environ, startResponse):
		cookies = get_cookies(environ)
		for h in range(5):
			try:
				cursor = db.Cursor()
				cursor.Begin()
				session = Session.Load(cursor, cookies)
				cursor.Rollback()
				break
			except OperationalError:
				if h == 4:
					raise
		handlerList = self.Controllers[environ['REQUEST_METHOD']][::-1]

		formVars = parse_formvars(environ)
		formVars = MultiDict([(k, v)
							   for (k, v) in list(formVars.items())])

		path = environ.get('PATH_INFO', '/')
		for cls in handlerList:
			pathMatch = cls.__path__.match(path)
			if not pathMatch:
				continue

			fullVars = copy.copy(formVars)
			fullVars.update(pathMatch.groupdict())

			cursor.begin()
			handler = cls(self, environ, startResponse, session, cursor)

			injector = self.InjectorFactory()
			injector['formVars'] = formVars.items()
			injector['currentUser'] = handler.CurrentUser
			injector['databaseCursor'] = cursor

			try:
				response = injector(handler.Handle)
			except HTTPStatus as response:
				headers = []
				for cookie in list(cookies.values()):
					cookie['path'] = '/'
					headers.append(('Set-cookie', cookie.OutputString()))
				if hasattr(response, 'Location'):
					headers.append(('Location', response.Location))
				headers.extend(handler.Headers)
				startResponse(HTTP_CODES[response.StatusCode], headers)
				Session.Save(cursor, cookies, session)
				return ()
			if response is None:
				cursor.Rollback()
				continue
			handler.SendHeaders(cookies)
			Session.Save(cursor, cookies, session)
			if isinstance(response, bytes):
				return [response]
			elif isinstance(response, str):
				return [response.encode('utf-8')]
			else:
				return response
		# No handlers available for this path.
		startResponse(HTTP_CODES[404], [])
		return ()

	def GetTemplate(self, name):
		if ':' in name:
			name, definition = name.split(':')
		else:
			definition = None
		template = self.TemplateLookup.get_template(name + '.mako')
		if definition:
			template = template.get_def(definition)
		return template

application = Application()

class HandlerType (type):
	def __new__(mcs, name, bases, classDict):
		if name == 'Handler':
			return type.__new__(mcs, name, bases, classDict)

		if classDict.get('__method__') not in ('GET', 'POST', 'OPTIONS'):
			raise ValueError('Invalid __method__ for ' + name)
		if '__path__' not in classDict:
			raise ValueError('No __path__ for ' + name)

		if classDict['__path__'] is None:
			classDict['__path__'] = re.compile('^.*$')
		else:
			classDict['__path__'] = re.compile('^' + classDict['__path__'] + '$')

		cls = type.__new__(mcs, name, bases, classDict)

		application.Controllers[classDict['__method__']].append(cls)

		return cls

class Handler (object, metaclass=HandlerType):
	_base_dir = sys.path[0]

	def __init__(self, application, environ, startResponse, session, databaseCursor):
		self.Application = application
		self.Environment = environ
		self.StartResponse = startResponse
		self.Session = session
		self.DatabaseCursor = databaseCursor
		self._ManagerCache = {}
		self.Model = DBRow()
		self.StatusCode = 200
		self.ContentType = 'text/html'
		self.Headers = []

		if 'paste.parsed_querystring' in self.Environment:
			self.path = self.Environment.get('PATH_INFO', '/') + '?' + self.Environment['paste.parsed_querystring'][1]
		else:
			self.path = self.Environment.get('PATH_INFO', '/')

		from User.Business.UserManager import UserManager
		self.CurrentUser = None
		try:
			self.CurrentUser = self.GetManager(UserManager).GetById(getattr(self.Session, 'user', None))
		except ValueError:
			pass

	def GetManager(self, managerType):
		if managerType not in self._ManagerCache:
			self._ManagerCache[managerType] = managerType(currentUser = self.CurrentUser,
														  databaseCursor = self.DatabaseCursor)
		return self._ManagerCache[managerType]

	def Handle(self, **kwargs):
		"""Handle this request.

		Returns None if this handler cannot take this request. Otherwise,
		return an iterable with the (possibly empty) response."""
		raise NotImplementedError

	def SendHeaders(self, cookies):
		if self.StatusCode == 200:
			self.Headers.append(('Content-type', self.ContentType + '; charset=utf-8'))
		for cookie in list(cookies.values()):
			cookie['path'] = '/'
			self.Headers.append(('Set-cookie', cookie.OutputString()))
		self.StartResponse(HTTP_CODES[self.StatusCode], self.Headers)

	def RenderTemplate(self, name, *args, **kwargs):
		template = self.Application.GetTemplate(name)
		return template.render(*args,
							   session = self.Session,
							   user = self.CurrentUser,
							   path = self.path,
							   config = config,
							   Model = self.Model,
							   **kwargs)

	def HasPermission(self, path, permission):
		if self.CurrentUser is None:
			return False
		return self.CurrentUser.HasPermission(path, permission)

	def RequirePermission(self, path, permission):
		self.RequireLogin()
		if not self.CurrentUser.HasPermission(path, permission):
			raise Forbidden()

	def IsLoggedIn(self):
		return self.CurrentUser is not None

	def RequireLogin(self):
		if not self.IsLoggedIn():
			raise SeeOther('/user/=/login/', {'callback': self.path})

class Session (object):
	def __init__(self):
		self._data = {}
		self._modified = False

	@classmethod
	def New(cls, cursor, cookies):
		if 'session_id' in cookies:
			del cookies['session_id']
		cookies['session_id'] = str(uuid.uuid4())
		cursor.Execute('''
			INSERT INTO sessions (uuid, dict)
			VALUES (%s, %s)''',
			(cookies['session_id'].value,
			 pickle.dumps({})))
		cursor.Commit()
		return cls()

	@classmethod
	def Load(cls, cursor, cookies):
		if 'session_id' not in cookies:
			return cls.New(cursor, cookies)
		try:
			sessionRow = cursor.Single('''
				SELECT dict
				FROM sessions
				WHERE uuid = %s''',
				(cookies['session_id'].value, ))
		except ValueError:
			return cls.New(cursor, cookies)
		try:
			session = cls()
			session._data = pickle.loads(sessionRow['dict'])
		except pickle.UnpicklingError:
			return cls.New(cursor, cookies)
		return session

	@classmethod
	def Save(cls, cursor, cookies, session):
		if not session._modified:
			return
		assert 'session_id' in cookies
		cursor.Execute('''
			UPDATE sessions
			SET dict = %s
			WHERE uuid = %s''',
			(pickle.dumps(session._data),
			 cookies['session_id'].value))
		cursor.Commit()

	def __getattr__(self, key):
		try:
			return self._data[key]
		except KeyError:
			raise AttributeError('%r object has no attribute %r' % (type(self).__name__, key))

	def __setattr__(self, key, value):
		if key.startswith('_'):
			self.__dict__[key] = value
		else:
			self._data[key] = value
			self._modified = True

	def __delattr__(self, key):
		assert not key.startswith('_')
		del self._data[key]
		self._modified = True

class OptionsHandler (Handler):
	__method__ = 'OPTIONS'
	__path__ = None

	def Handle(self):
		self.Headers.append(('Allow', 'GET,POST,OPTIONS'))
		return []


class EntityManagerType (type):
	def __new__(mcs, name, bases, classDict):
		if name == 'EntityManager':
			return type.__new__(mcs, name, bases, classDict)

		cls = type.__new__(mcs, name, bases, classDict)

		if '__entity__' in classDict:
			application.Manager[classDict['__entity__']] = cls

		return cls

class EntityManager (metaclass = EntityManagerType):
	def __init__(self, *, currentUser, databaseCursor):
		self.CurrentUser = currentUser
		self.DatabaseCursor = databaseCursor
