import threading
import pymysql

import lawrence.config as config
from lawrence.util import Deprecated

class DBRow (dict):
    def __repr__(self):
        return repr(dict(list(self.items())))

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            if not key.endswith('_'):
                raise
            return self[key[:-1]]

    def __setattr__(self, key, value):
        self[key] = value

    def __bool__(self):
        return len([x for x in list(self.values()) if x is not None]) > 0

class Connection (threading.local):
    database = None
    def __init__(self, *args, **kwargs):
        threading.local.__init__(self, *args, **kwargs)
        self.database = pymysql.connect(
            host = config.db_host,
            port = 3306,
            db = config.db_name,
            user = config.db_user,
            passwd = config.db_password,
            charset = 'utf8')
CONNECTION = Connection()

class Cursor (object):
    def __init__(self):
        self._cursor = CONNECTION.database.cursor(pymysql.cursors.DictCursor)

    def Execute(self, *args, **kwargs):
        self._cursor.execute(*args, **kwargs)
        resultSet = [DBRow(row) for row in self._cursor.fetchall()]
        return resultSet

    def Single(self, *args, **kwargs):
        resultSet = self.Execute(*args, **kwargs)
        if len(resultSet) == 1:
            return resultSet[0]
        elif len(resultSet) == 0:
            raise ValueError  # FIXME: add error message “no rows returned”
        else:
            raise ValueError  # FIXME: add error message “more than one row returned”

    def Begin(self):
        self.Execute('BEGIN')

    def Commit(self):
        self.Execute('COMMIT')

    def Rollback(self):
        self.Execute('ROLLBACK')

    @Deprecated
    def begin(self):
        self.Begin()

    def ScopeIdentity(self):
        return self.Single('SELECT LAST_INSERT_ID() AS Id').Id

@Deprecated
def Execute(*args, **kwargs):
    cursor = Cursor()
    result = cursor.Execute(*args, **kwargs)
    cursor.Commit()
    return result

@Deprecated
def Single(*args, **kwargs):
    cursor = Cursor()
    return cursor.Single(*args, **kwargs)

def query_in(field, seq):
    """Return a MySQL condition suitable for doing a set incidence query.

    Using None in place of a sequence returns a tautology."""
    if seq is None:
        return ' (1 = 1) '
    if len(seq) == 0:
        return ' (1 = 0) '
    if None in seq:
        seq = [x for x in seq if x is not None]
        return ' (%s IS NULL OR (%s)) ' % (field, query_in(field, seq))
    # FIXME: this breaks when seq contains Unicode strings.
    return ' (%s IN (%s)) ' % (field, ', '.join(
        CONNECTION.database.escape(item) for item in seq))

def UnicodeEscape(s):
    return '\'' + s.replace('\'', '\'\'') + '\''
