try:
    import cv
except ImportError:
    cv = None
import os
import tempfile
import lawrence.base

def DetectFace(imageData):
    if cv is None:
        return None

    _, imagePath = tempfile.mkstemp('.jpg')
    with open(imagePath, 'wb') as imageFile:
        imageFile.write(imageData)
    image = cv.LoadImage(imagePath)
    os.unlink(imagePath)
    imageSize = cv.GetSize(image)

    # create grayscale version
    grayscale = cv.CreateImage(imageSize, 8, 1)
    cv.CvtColor(image, grayscale, cv.CV_BGR2GRAY)

    # create storage
    storage = cv.CreateMemStorage(0)

    # equalize histogram
    cv.EqualizeHist(grayscale, grayscale)

    # detect objects
    cascade = cv.Load(os.path.join(lawrence.base.Handler._base_dir,
                                   'haarcascade_frontalface_alt.xml'))
    faces = cv.HaarDetectObjects(grayscale, cascade, storage, 1.2, 2,
                                 cv.CV_HAAR_DO_CANNY_PRUNING, (50, 50))

    if not faces:
        return None

    class FaceRectangle (object):
        def __init__(self, xOffset, yOffset, width, height):
            self.XOffset = xOffset
            self.YOffset = yOffset
            self.Width = width
            self.Height = height

        def Area(self):
            return self.Width * self.Height
    faces = [FaceRectangle(xOffset, yOffset, width, height) for (xOffset, yOffset, width, height), _ in faces]
    largestFace = max(faces, key = lambda face: face.Area())

    sw = largestFace.Width / 2.0
    sh = largestFace.Height / 2.0
    cx = largestFace.XOffset + sw
    cy = largestFace.YOffset + sh

    # clip rectangle to image
    rect = [int(x + 0.5) for x in (cx - sw, cy - sh, 2 * sw, 2 * sh)]
    if rect[0] < 0:
        rect[2] += rect[0]
        rect[0] = 0
    if rect[1] < 0:
        rect[3] += rect[1]
        rect[1] = 0
    rect[2] = min(rect[2], imageSize[0] - rect[0])
    rect[3] = min(rect[3], imageSize[1] - rect[1])
    rect = tuple(rect)

    # scale to final size
    face = cv.GetSubRect(image, rect)
    scaledFace = cv.CreateMat(120, 120, cv.CV_8UC3)
    cv.Resize(face, scaledFace)

    # extract image data
    _, facePath = tempfile.mkstemp('.jpg')
    cv.SaveImage(facePath, scaledFace)
    with open(facePath, 'rb') as faceFile:
        faceData = faceFile.read()

    return faceData
