(function () {
	function ToArray(obj) {
		var array = [];
		for (var i = 0; obj[i] !== undefined; i++) {
			array.push(obj[i]); }
		return array; }

    var QueryInput = document.getElementById('Query');
    var FilterList = [].map.call(
        document.querySelectorAll('table[data-filter]'),
        function (table) {
            return {
                Name: table.dataset.filter,
                RadioList: ToArray(table.querySelectorAll('input')),
                GetChecked: function () {
                    var checkedValues = [];
                    for (var i = 0; i < this.RadioList.length; i++) {
                        if (this.RadioList[i].checked && this.RadioList[i].value) {
                            checkedValues.push(this.RadioList[i].value);
                        }
                    }
                    // TODO: choose behaviour depending on whether we are on radio or checkbox mode
                    return checkedValues.length > 0 ? checkedValues : null;
                }}});
    var pageFilter = {
        Name: 'Page',
        Input: $('div[data-filter=Page] input')[0],
        PreviousPageButton: $('div[data-filter=Page] .PagePrevious')[0],
        PageNumber: $('div[data-filter=Page] .PageNumber')[0],
        NextPageButton: $('div[data-filter=Page] .PageNext')[0],
        GetChecked: function () { return [this.Input.value]; }};  // FIXME
    FilterList.push(pageFilter);
    var pageCount = null;  // FIXME

    function updatePageButtons() {
        if ((pageFilter.Input.value || 1) >= 2) {
            pageFilter.PreviousPageButton.classList.remove('muted'); }
        else {
            pageFilter.PreviousPageButton.classList.add('muted'); }

        if ((pageFilter.Input.value || 1) <= pageCount - 1) {
            pageFilter.NextPageButton.classList.remove('muted'); }
        else {
            pageFilter.NextPageButton.classList.add('muted'); }}

    function previousPage() {
        if ((pageFilter.Input.value || 1) <= 1) return;
        pageFilter.Input.value--;
        pageFilter.PageNumber.innerText--;
        updatePageButtons();
        updateIndex(); }

    function nextPage() {
        if ((pageFilter.Input.value || 1) >= pageCount) return;
        pageFilter.Input.value++;
        pageFilter.PageNumber.innerText++;
        updatePageButtons();
        updateIndex(); }

    function getFilterDict() {
        var filterDict = {};
        FilterList.forEach(function (filter) {
            var filterValue = filter.GetChecked();
            if (filterValue !== null) {
                filterDict[filter.Name] = filterValue; }});
        if (QueryInput.value) {
            filterDict.Query = QueryInput.value; }
        return filterDict; }

    window.getFilterDict = getFilterDict;

    function openFilter(filterName) {
        var filterHeader = $('h3[data-filter="' + filterName + '"]');
        filterHeader.next('.box').slideDown(400, function () { }); }

    function closeFilter(filterName) {
        var filterHeader = $('h3[data-filter="' + filterName + '"]');
        filterHeader.next('.box').slideUp(400, function () { }); }

    function removeFilter(filterName) {
        var filterTable = $('table[data-filter="' + filterName + '"]');
        filterTable.find('input').prop('checked', false);
        closeFilter(filterName);
        updateFilterHeader(filterName); }

    function getRadioLabel(radio) {
        return $(radio).closest('td').next('td').text(); }

    function updateFilterHeader(filterName) {
        var filterDict = getFilterDict();

        var filterTable = $('table[data-filter="' + filterName + '"]');
        var selectionLabels = [].concat(filterDict[filterName]).map(function (value) {
            return getRadioLabel(filterTable.find('input[value="' + value + '"]')[0]);
        }).join(", ");
        var filterHeader = $('h3[data-filter="' + filterName + '"]');
        filterHeader.toggleClass('filter-active', filterDict[filterName] !== undefined);
        filterHeader.find('.FilterSelection').text(selectionLabels);
        var btnRemove = filterHeader.find('.btnRemove');
        btnRemove.remove();
        if (filterHeader.hasClass('filter-active')) {
            btnRemove = $('<a class="btnRemove">&#xF00D;</a>');
            filterHeader.append(btnRemove);
            btnRemove.click(function () {
                removeFilter(filterName);
				updateIndex();
                return false; }); }}

    function updateFilters() {
        var filterDict = getFilterDict();

        FilterList.forEach(function (filter) {
            updateFilterHeader(filter.Name); });

        $.ajax({
                url: '/Candidate/_FilterCounts/',
                type: 'GET',
                data: queryStringFromObject(filterDict)})
            .success(function (data) {
                for (var filterName in data) {
                    if (!data.hasOwnProperty(filterName)) {
                        continue; }
                    var filterCounts = data[filterName];
                    var filterTable = $('table[data-filter="' + filterName + '"]');
                    if (filterTable.length) {
                        $.makeArray(filterTable.find('tr')).forEach(function (row) {
                            var $row = $(row);
                            var rowInput = $row.find('input[type="radio"], input[type="checkbox"]');
                            var rowBadge = $row.find('span.badge');
                            var rowCount = filterCounts[rowInput.val()];
                            $row.toggleClass('disabled', rowCount === undefined);
                            rowInput.attr('disabled', rowCount === undefined);
                            rowBadge.toggle(rowCount !== undefined);
                            rowBadge.text(rowCount || 0); }); }}

                if (data.PageCount !== 0) {
                    pageCount = data.PageCount;
                    updatePageButtons(); }}); }

    function queryStringFromObject(obj) {
        var parts = [];
        for (var part in obj) {
            if (obj.hasOwnProperty(part)) {
                var valueList = [].concat(obj[part]);
                for (var i = 0; i < valueList.length; i++) {
                    var value = valueList[i];   
                    parts.push(encodeURIComponent(part) + '=' + encodeURIComponent(value)); }}}
        return parts.join('&'); }

    window.queryStringFromObject = queryStringFromObject;

    function updateIndex() {
        var filterDict = getFilterDict();
        var filterQueryString = queryStringFromObject(filterDict);

        if (filterDict.Status !== undefined && filterDict.Status.length === 1 && filterDict.Status[0] === 'present') {
            $('#AppointmentStatusGroup').show(); }
        else {
			removeFilter('AppointmentStatus');
            $('#AppointmentStatusGroup').hide(); }

        var $spinner = CreateSpinner($('#CandidateListing'));
        $.ajax({
                url: '/Candidate/_Listing/',
                type: 'GET',
                data: queryStringFromObject(filterDict)
            }).success(function (data) {
                $('#CandidateListing').html(data);
                DestroySpinner($spinner);
                window.history.replaceState(filterDict, '', window.location.origin + window.location.pathname + '?' + filterQueryString); });

        updateFilters(); }

    document.addEventListener('DOMContentLoaded', function () {
        pageFilter.PreviousPageButton.addEventListener('click', previousPage, true);
        pageFilter.NextPageButton.addEventListener('click', nextPage, true);

        // TODO: setup backend typeahead.
/*
        $(QueryInput).typeahead({
            source: function (query, process) {
                return $.ajax({
                    url: '/candidate/',
                    type: 'GET',
                    data: {ajax: 1, query: query}, // FIXME: use current filters
                }).success(function (data, textSuccess, jqXHR) {
                    data.unshift(query);
                    process(data);
                });
            },
            updater: function (item) {
                QueryInput.value = item;
                updateIndex();
                return item;
            }
        });
*/
        $(QueryInput).keypress(function (e) {
			if (e.charCode == 13) {
				updateIndex();

				e.preventDefault();
				e.stopPropagation(); }});

        document.getElementById('QueryButton').addEventListener('click', updateIndex);

        FilterList.forEach(function (filter) {
            var filterTable = $('table[data-filter="' + filter.Name + '"]');
            var filterHeader = $('h3[data-filter="' + filter.Name + '"]')[0];
            var filterBox = $(filterHeader).next('.box')[0];
            var btnToggleMultiSelect = $(filterTable).prev('.btnToggleMultiSelect')[0];

			if (filter.RadioList !== undefined) {
				filter.RadioList.forEach(function (radio) {
					radio.addEventListener('change', function () {
						if (this.type == 'radio') {
							closeFilter(filter.Name);
						}
						updateIndex();
					}); }); }

			if (filterHeader !== undefined) {
				filterHeader.addEventListener('click', function () {
					if (!$(filterBox).is(':visible')) {
						openFilter(filter.Name); }
                    else {
						closeFilter(filter.Name); }
					return false; }); }

            if (btnToggleMultiSelect !== undefined) {
				var imgToggleMultiSelect = $(btnToggleMultiSelect).find('i')[0];
				btnToggleMultiSelect.addEventListener('click', function () {
					$(imgToggleMultiSelect).toggleClass('icon-check').toggleClass('icon-circle-blank');
					filter.RadioList.forEach(function (radio) { radio.type = (radio.type === 'checkbox') ? 'radio' : 'checkbox'; });
					updateIndex(); }); }});

        if (history.state) {
            // TODO: populate radio/checkboxes
            pageFilter.PageNumber.innerText = history.state.Page || 1;
            pageFilter.Input.value = history.state.Page || 1; }

        updatePageButtons();
        updateFilters(); });
})();
